/*
* Creaci de les taules de la BD
* S'ha d'executar connectat com a monitor/monitor1606
*/

spool 03_CreacioTaules.log;

-- Taula CLIENT.
CREATE TABLE CLIENT
(
  ID_CLIENT				NUMBER(10)      NOT NULL,
  DATA_ALTA       DATE	  				NOT NULL,
  TIPUS_PERSONA   CHAR(1)					NOT NULL,
  NOM         		VARCHAR2(100)		NOT NULL,
  PRIMER_COGNOM   VARCHAR2(50),
  SEGON_COGNOM    VARCHAR2(50),
  TIPUS_DOCUMENT  CHAR(1)					NOT NULL,
  CODI_DOCUMENT   VARCHAR2(20)		NOT NULL,
  TELEFON_FIX   	VARCHAR2(20),
  TELEFON_MOBIL   VARCHAR2(20),
  EMAIL	        	VARCHAR2(40)		NOT NULL,
  CONSTRAINT	PK_CLIENT 					PRIMARY KEY(ID_CLIENT),
  CONSTRAINT 	UK_CLIENT 					UNIQUE(CODI_DOCUMENT),
  CONSTRAINT 	CK1_CLIENT 					CHECK (TIPUS_PERSONA IN('F','J')),
  CONSTRAINT 	CK2_CLIENT 					CHECK (TIPUS_DOCUMENT IN('C','N','E','A'))
)
TABLESPACE TBS_MONITOR;

-- Taula SERVEI_WEB.
CREATE TABLE SERVEI_WEB
(
  ID_SERVEI_WEB		NUMBER(10)      	NOT NULL,
  CLIENT					NUMBER(10)      	NOT NULL,
  DESCRIPCIO   		VARCHAR2(150)			NOT NULL,
  DATA_ALTA       DATE	  					NOT NULL,
  TIPUS_SERVEI	  CHAR(1)						NOT NULL,
  URL					    VARCHAR2(200)			NOT NULL,
  ESTAT					  CHAR(1)						NOT NULL,
  DATA_ESTAT      DATE	  					NOT NULL,
  CONSTRAINT 	PK_ID_SERVEI_WEB 			PRIMARY KEY(ID_SERVEI_WEB),
  CONSTRAINT	FK_SERVEI_WEB_CLIENT 	FOREIGN KEY (CLIENT) REFERENCES CLIENT(ID_CLIENT),
  CONSTRAINT 	UK_SERVEI_WEB 				UNIQUE(CLIENT, DESCRIPCIO),
  CONSTRAINT 	CK1_SERVEI_WEB 				CHECK(TIPUS_SERVEI IN('H','R','S')),
  CONSTRAINT 	CK2_SERVEI_WEB 				CHECK(ESTAT IN('A','B','D','N'))
)
TABLESPACE TBS_MONITOR;

-- Taula HIST_ESTAT_SERVEI_WEB.
CREATE TABLE HIST_ESTAT_SERVEI_WEB
(
  ID_HIST_ESTAT_SERVEI_WEB	NUMBER(10)      	NOT NULL,
  SERVEI_WEB								NUMBER(10)      	NOT NULL,
  DATA_INICI      					DATE	  					NOT NULL,
  DATA_FI      							DATE	  					NOT NULL,
  ESTAT					  					CHAR(1)						NOT NULL,
  CONSTRAINT	PK_HIST_ESTAT_SERVEI_WEB 				PRIMARY KEY(ID_HIST_ESTAT_SERVEI_WEB),
  CONSTRAINT 	FK_HIST_ESTAT_SERVEI_WEB_SWEB 	FOREIGN KEY(SERVEI_WEB) REFERENCES SERVEI_WEB(ID_SERVEI_WEB),
  CONSTRAINT 	UK_HIST_ESTAT_SERVEI_WEB 				UNIQUE(SERVEI_WEB, DATA_INICI, DATA_FI),
  CONSTRAINT 	CK1_HIST_ESTAT_SERVEI_WEB 			CHECK(DATA_FI > DATA_INICI),
  CONSTRAINT 	CK2_HIST_ESTAT_SERVEI_WEB 			CHECK(ESTAT IN('A','B','D','N'))
)
TABLESPACE TBS_MONITOR;

-- Taula REGLA.
CREATE TABLE REGLA
(
  ID_REGLA				NUMBER(10)    		NOT NULL,
  DATA_ALTA       DATE	  					NOT NULL,
  SERVEI_WEB			NUMBER(10)   		 	NOT NULL,
  NOM				   		VARCHAR2(150)			NOT NULL,
  PERIODICITAT		NUMBER(10)    		NOT NULL,
  ESTAT					  CHAR(1)						NOT NULL,
  DATA_ESTAT      DATE	  					NOT NULL,
  CONSTRAINT 	PK_ID_REGLA 					PRIMARY KEY(ID_REGLA),
  CONSTRAINT	FK_REGLA_SERVEI_WEB 	FOREIGN KEY (SERVEI_WEB) REFERENCES SERVEI_WEB(ID_SERVEI_WEB),
  CONSTRAINT 	UK_REGLA 							UNIQUE(SERVEI_WEB, NOM),
  CONSTRAINT 	CK_REGLA 							CHECK(ESTAT IN('A','B','D'))
)
TABLESPACE TBS_MONITOR;

-- Taula HIST_ESTAT_REGLA.
CREATE TABLE HIST_ESTAT_REGLA
(
  ID_HIST_ESTAT_REGLA	NUMBER(10)      		NOT NULL,
  REGLA								NUMBER(10)      		NOT NULL,
  DATA_INICI      		DATE	  						NOT NULL,
  DATA_FI      				DATE	  						NOT NULL,
  ESTAT					  		CHAR(1)							NOT NULL,
  CONSTRAINT	PK_HIST_ESTAT_REGLA 				PRIMARY KEY(ID_HIST_ESTAT_REGLA),
  CONSTRAINT 	FK_HIST_ESTAT_REGLA_REGLA 	FOREIGN KEY(REGLA) REFERENCES REGLA(ID_REGLA),
  CONSTRAINT 	UK_HIST_ESTAT_REGLA 				UNIQUE(REGLA, DATA_INICI, DATA_FI),
  CONSTRAINT 	CK1_HIST_ESTAT_REGLA 				CHECK(DATA_FI > DATA_INICI),
  CONSTRAINT 	CK2_HIST_ESTAT_REGLA 				CHECK(ESTAT IN('A','B','D'))
)
TABLESPACE TBS_MONITOR;

-- Taula PAS.
CREATE TABLE PAS
(
  ID_PAS						NUMBER(10)    		NOT NULL,
  REGLA							NUMBER(10)   		 	NOT NULL,
  ORDRE							NUMBER(5)   		 	NOT NULL,
  DATA_ALTA     		DATE	  					NOT NULL,
  DESCRIPCIO   			VARCHAR2(150)			NOT NULL,
  URL_SERVEI    		VARCHAR2(200)			NOT NULL,
  TEMPS_MAXIM				NUMBER(10)    		NOT NULL,
  PETICIO_SERVEI   	VARCHAR2(250)			NOT NULL,
  RESPOSTA_ESPERADA	VARCHAR2(250)			NOT NULL,
  ESTAT							CHAR(1)						NOT NULL,
  DATA_ESTAT    		DATE	  					NOT NULL,
  CONSTRAINT 	PK_ID_PAS 							PRIMARY KEY(ID_PAS),
  CONSTRAINT	FK_PAS_REGLA			 			FOREIGN KEY (REGLA) REFERENCES REGLA(ID_REGLA),
  CONSTRAINT 	UK_PAS 									UNIQUE(REGLA, ORDRE),
  CONSTRAINT 	CK_PAS 									CHECK(ESTAT IN('A','B','D'))
)
TABLESPACE TBS_MONITOR;

-- Taula HIST_ESTAT_PAS.
CREATE TABLE HIST_ESTAT_PAS
(
  ID_HIST_ESTAT_PAS		NUMBER(10)      	NOT NULL,
  PAS									NUMBER(10)      	NOT NULL,
  DATA_INICI      		DATE	  					NOT NULL,
  DATA_FI      				DATE	  					NOT NULL,
  ESTAT					  		CHAR(1)						NOT NULL,
  CONSTRAINT	PK_HIST_ESTAT_PAS 				PRIMARY KEY(ID_HIST_ESTAT_PAS),
  CONSTRAINT 	FK_HIST_ESTAT_PAS_PAS 		FOREIGN KEY(PAS) REFERENCES PAS(ID_PAS),
  CONSTRAINT 	UK_HIST_ESTAT_PAS 				UNIQUE(PAS, DATA_INICI, DATA_FI),
  CONSTRAINT 	CK1_HIST_ESTAT_PAS 				CHECK(DATA_FI > DATA_INICI),
  CONSTRAINT 	CK2_HIST_ESTAT_PAS 				CHECK(ESTAT IN('A','B','D'))
)
TABLESPACE TBS_MONITOR;

-- Taula REGISTRE.
CREATE TABLE REGISTRE
(
  ID_REGISTRE					NUMBER(10)    		NOT NULL,
  SERVEI_WEB					NUMBER(10)   		 	NOT NULL,
  REGLA								NUMBER(10)   		 	NOT NULL,
  DATA_INICI     			DATE	  					NOT NULL,
  DATA_FI     				DATE	  					,
  TEMPS_RESPOSTA_TOT	NUMBER(10)    		,
  ULTIM_PAS						NUMBER(10)    		,
  IND_DISPONIBLE			CHAR(1)						NOT NULL,
  CODI_ERROR    			VARCHAR2(3)	  		,
  CONSTRAINT 	PK_ID_REGISTRE 						PRIMARY KEY(ID_REGISTRE),
  CONSTRAINT	FK_REGISTRE_SERVEI_WEB		FOREIGN KEY (SERVEI_WEB) REFERENCES SERVEI_WEB(ID_SERVEI_WEB),
  CONSTRAINT	FK_REGISTRE_REGLA			 		FOREIGN KEY (REGLA) REFERENCES REGLA(ID_REGLA),
  CONSTRAINT	FK_REGISTRE_PAS			 			FOREIGN KEY (ULTIM_PAS) REFERENCES PAS(ID_PAS),
  CONSTRAINT 	UK_REGISTRE 							UNIQUE(SERVEI_WEB, REGLA, DATA_INICI),
  CONSTRAINT 	CK1_REGISTRE 							CHECK(DATA_FI > DATA_INICI),
  CONSTRAINT 	CK2_REGISTRE			 				CHECK(IND_DISPONIBLE IN('E','S','N'))
)
TABLESPACE TBS_MONITOR;

-- Taula LINIA_REGISTRE.
CREATE TABLE LINIA_REGISTRE
(
  ID_LINIA_REGISTRE	NUMBER(10)    		NOT NULL,
  REGISTRE					NUMBER(10)   		 	NOT NULL,
  PAS								NUMBER(10)   		 	NOT NULL,
  DATA_INICI     		DATE	  					NOT NULL,
  DATA_FI     			DATE	  					NOT NULL,
  TEMPS_RESPOSTA		NUMBER(10)    		NOT NULL,
  IND_DISPONIBLE		CHAR(1)						NOT NULL,
  CODI_ERROR    		VARCHAR2(3)	  		,
  CONSTRAINT 	PK_ID_LINIA_REGISTRE 		PRIMARY KEY(ID_LINIA_REGISTRE),
  CONSTRAINT	FK_LINIA_REGISTRE_PAS		FOREIGN KEY (PAS) REFERENCES PAS(ID_PAS),
  CONSTRAINT 	UK_LINIA_REGISTRE 			UNIQUE(REGISTRE, PAS, DATA_INICI),
  CONSTRAINT 	CK1_LINIA_REGISTRE 			CHECK(DATA_FI > DATA_INICI),
  CONSTRAINT 	CK2_LINIA_REGISTRE			CHECK(IND_DISPONIBLE IN('S','N'))
)
TABLESPACE TBS_MONITOR;

-- Taula DISPONIBILITAT_MES.
CREATE TABLE DISPONIBILITAT_MES
(
  ID_DISPONIBILITAT_MES	    NUMBER(10)    NOT NULL,
  ANYO     					    		NUMBER(4)   	NOT NULL,
  MES												NUMBER(2)   	NOT NULL,
  DATA_CALCUL    		        DATE	  			NOT NULL,
  SERVEI_WEB    		        NUMBER(10)    NOT NULL,
  REGLA         		        NUMBER(10)		NOT NULL,
  REGISTRES_TOT  		        NUMBER(10)		DEFAULT 0 NOT NULL,
  REGISTRES_DISP 		        NUMBER(10)		DEFAULT 0 NOT NULL,
  REGISTRES_NO_DISP	        NUMBER(10)		DEFAULT 0 NOT NULL,
  PERCENTATGE_DISP          FLOAT     		DEFAULT 0 NOT NULL,
  TEMPS_EXECUCIO_TOT 		    NUMBER(10)		DEFAULT 0 NOT NULL,
  TEMPS_EXECUCIO_DISP	      NUMBER(10)		DEFAULT 0 NOT NULL,
  TEMPS_EXECUCIO_MIG_DISP   FLOAT     		DEFAULT 0 NOT NULL,
  CONSTRAINT 	PK_DISPONIBILITAT_MES 							PRIMARY KEY(ID_DISPONIBILITAT_MES),
  CONSTRAINT	FK_DISPONIBILITAT_MES_SERVEI				FOREIGN KEY (SERVEI_WEB) REFERENCES SERVEI_WEB(ID_SERVEI_WEB),
  CONSTRAINT	FK_DISPONIBILITAT_MES_REGLA		    	FOREIGN KEY (REGLA) REFERENCES REGLA(ID_REGLA),
  CONSTRAINT 	UK_DISPONIBILITAT_MES     					UNIQUE(ANYO, MES, SERVEI_WEB, REGLA)
)
TABLESPACE TBS_MONITOR;

-- Taula DISPONIBILITAT_DIA.
CREATE TABLE DISPONIBILITAT_DIA
(
  ID_DISPONIBILITAT_DIA	    NUMBER(10)    NOT NULL,
  ANYO     					    		NUMBER(4)   	NOT NULL,
  MES												NUMBER(2)   	NOT NULL,
  DIA												NUMBER(2)   	NOT NULL,
  DATA_CALCUL    		        DATE	  			NOT NULL,
  REGISTRES_TOT  		        NUMBER(10)		DEFAULT 0 NOT NULL,
  REGISTRES_DISP 		        NUMBER(10)		DEFAULT 0 NOT NULL,
  REGISTRES_NO_DISP	        NUMBER(10)		DEFAULT 0 NOT NULL,
  PERCENTATGE_DISP          FLOAT     		DEFAULT 0 NOT NULL,
  CONSTRAINT 	PK_DISPONIBILITAT_DIA 			PRIMARY KEY(ID_DISPONIBILITAT_DIA),
  CONSTRAINT 	UK_DISPONIBILITAT_DIA     	UNIQUE(ANYO, MES, DIA)
)
TABLESPACE TBS_MONITOR;

-- Taula DW_DISP_SERVEI_TRAM.
CREATE TABLE DW_DISP_SERVEI_TRAM
(
  ID_DW_DISP_SERVEI_TRAM	NUMBER(10)  NOT NULL,
  ANYO     								NUMBER(4)   NOT NULL,
  MES											NUMBER(2)   NOT NULL,
  DATA_CALCUL    					DATE	  		NOT NULL,
  IND_TRAM  							CHAR(1)			NOT NULL,
  NOMBRE_SERVEIS					NUMBER(10)	DEFAULT 0 NOT NULL,
  CONSTRAINT 	PK_DW_DISP_SERVEI_TRAM 	PRIMARY KEY(ID_DW_DISP_SERVEI_TRAM),
  CONSTRAINT 	UK_DW_DISP_SERVEI_TRAM  UNIQUE(ANYO, MES, IND_TRAM)
)
TABLESPACE TBS_MONITOR;

-- Taula DW_RESULTAT_DISP.
CREATE TABLE DW_RESULTAT_DISP
(
  ID_DW_RESULTAT_DISP	NUMBER(10)  NOT NULL,
  ANYO     						NUMBER(4)   NOT NULL,
  MES									NUMBER(2)   NOT NULL,
  DATA_CALCUL    			DATE	  		NOT NULL,
  IND_DISPONIBLE  		CHAR(1)			NOT NULL,
  CODI_ERROR  				VARCHAR(3)	,
  NOMBRE_REGISTRES		NUMBER(10)	DEFAULT 0 NOT NULL,
  CONSTRAINT 	PK_DW_RESULTAT_DISP PRIMARY KEY(ID_DW_RESULTAT_DISP),
  CONSTRAINT 	UK_DW_RESULTAT_DISP UNIQUE(ANYO, MES, IND_DISPONIBLE, CODI_ERROR)
)
TABLESPACE TBS_MONITOR;

-- Taula DW_DIA_MENYS_DISP.
CREATE TABLE DW_DIA_MENYS_DISP
(
  ID_DW_DIA_MENYS_DISP	NUMBER(10)  NOT NULL,
  ANYO     						NUMBER(4)   		NOT NULL,
  MES									NUMBER(2)   		NOT NULL,
  DIA									NUMBER(2)   		NOT NULL,
  DATA_CALCUL    			DATE	  				NOT NULL,
  PERCENTATGE_DISP		FLOAT	DEFAULT 0 NOT NULL,
  CONSTRAINT 	PK_DW_DIA_MENYS_DISP PRIMARY KEY(ID_DW_DIA_MENYS_DISP)
)
TABLESPACE TBS_MONITOR;

-- Taula DW_CLIENT_MENYS_DISP.
CREATE TABLE DW_CLIENT_MENYS_DISP
(
  ID_DW_CLIENT_MENYS_DISP	NUMBER(10)  NOT NULL,
  ANYO     						NUMBER(4)   		NOT NULL,
  MES									NUMBER(2)   		NOT NULL,
  DATA_CALCUL    			DATE	  				NOT NULL,
  CLIENT     					NUMBER(10)   		NOT NULL,
  PERCENTATGE_DISP		FLOAT	DEFAULT 0 NOT NULL,
  CONSTRAINT 	PK_DW_CLIENT_MENYS_DISP 				PRIMARY KEY(ID_DW_CLIENT_MENYS_DISP),
  CONSTRAINT	FK_DW_CLIENT_MENYS_DISP_CLIENT 	FOREIGN KEY (CLIENT) REFERENCES CLIENT(ID_CLIENT)
)
TABLESPACE TBS_MONITOR;


-- Taula DW_REGLA_MES_TEMPS.
CREATE TABLE DW_REGLA_MES_TEMPS
(
  ID_DW_REGLA_MES_TEMPS	NUMBER(10)  NOT NULL,
  ANYO     						NUMBER(4)   NOT NULL,
  MES									NUMBER(2)   NOT NULL,
  DATA_CALCUL    			DATE	  		NOT NULL,
  REGLA     						NUMBER(10)   NOT NULL,
  TEMPS_MIG		FLOAT	DEFAULT 0 NOT NULL,
  CONSTRAINT 	PK_DW_REGLA_MES_TEMPS PRIMARY KEY(ID_DW_REGLA_MES_TEMPS),
  CONSTRAINT	FK_DW_REGLA_MES_TEMPS_REGLA 	FOREIGN KEY (REGLA) REFERENCES REGLA(ID_REGLA)
)
TABLESPACE TBS_MONITOR;


-- Taula LOG.
CREATE TABLE LOG
(
  ID_LOG          NUMBER(10)     	NOT NULL,
  DATA_HORA    	  DATE			 			NOT NULL,
  NOM_PROCEDIMENT VARCHAR2(50)  	NOT NULL,
  PARAM_ENTRADA	  VARCHAR2(400)  	,
  PARAM_SORTIDA	  VARCHAR2(250)  	,
  CONSTRAINT PK_LOG PRIMARY KEY (ID_LOG) 
)
TABLESPACE TBS_MONITOR;

spool off;