/*
* Creaci del package PKG_FUNCIONS.
* S'ha d'executar connectat com a monitor/monitor1606
*/

spool 09_PackageFuncions.log;

CREATE OR REPLACE PACKAGE PKG_FUNCIONS AS

 -- Funcio que retorna, per l'any i mes que rep com parametres,
 -- una llista de dies en els quals hi han hagut registres de 
 -- monitoritzacio que mostraven no dispoibilitat de serveis web.
 -- La informacio s'agrupa per servei web, regla i dia.
	FUNCTION FU_SERVEIS_NO_DISP_MES 	(P_ANY IN  NUMBER,
	   																 P_MES IN  NUMBER)
	RETURN TA_SERVEIS_NO_DISP_MES;
  
 -- Funcio que retorna, donat un nom de taula que rep com parametre
 -- d'entrada, l'identificador autoincremental de la fila que s'acava 
 -- de donar d'alta, com a consequencia de la crida a un procediment d'alta.
  FUNCTION FU_DB_GETCURRVAL          (P_NOM_TAULA  VARCHAR2)
  RETURN  NUMBER;

END PKG_FUNCIONS;
/


CREATE OR REPLACE PACKAGE BODY PKG_FUNCIONS AS

  -- DECLARACIO DE FUNCIONS PRIVADES.
  
  -- Funcio que retorna el temps estimat de disponibilitat (en hores)
  -- d'un servei web, en l'any i mes passats com a parametres.
  FUNCTION FU_TEMPS_DISP_SERVEI_MES   (P_SERVEI_WEB IN  NUMBER,
                                       P_ANY        IN  NUMBER,
                                       P_MES        IN  NUMBER)
  RETURN FLOAT;

  -- Funcio que retorna el % de disponibilitat d'un servei web en l'any 
  -- i mes passats com a parametres.
  FUNCTION FU_PERCENT_DISP_SERVEI_MES (P_SERVEI_WEB IN  NUMBER,
                                       P_ANY        IN  NUMBER,
                                       P_MES        IN  NUMBER)
  RETURN FLOAT;

/******************************************************************************
   Nom:                 FU_SERVEIS_NO_DISP_MES
   Propsit:            Funcio que retorna, per l'any i mes que rep com parametres,
                        una llista de dies en els quals hi han hagut registres de 
                        monitoritzacio que mostraven no dispoibilitat de serveis web.
                        La informacio s'agrupa per servei web, regla i dia.
                   
   Parmetres entrada:  -	P_ANY  NUMBER: any en que es vol fer la consulta.
                        -	P_MES  NUMBER: mes en que es vol fer la consulta. 

   Parmetres sortida:  -	RSP 
   Retorna:             -	si l'execuci ha finalitzat amb xit retorna una taula
                          amb la segnt informaci:
                          	el codi de contracte.     
                          	potncia mxima contractada. 
                          	percentatge de consum respecte al consum mitja.

                          NOTA: Si no es troba cap resultat la taula retornada 
                          estar buida.

                        - 'ERROR: tipus d'error' en cas de finalitzar amb error       
******************************************************************************/
FUNCTION FU_SERVEIS_NO_DISP_MES (P_ANY IN  NUMBER,
   															 P_MES IN  NUMBER)
RETURN TA_SERVEIS_NO_DISP_MES
IS

V_SERVEI_WEB_ANT    SERVEI_WEB.ID_SERVEI_WEB%TYPE := 0;
V_SERVEI_WEB        SERVEI_WEB.ID_SERVEI_WEB%TYPE;
V_REGLA              REGLA.ID_REGLA%TYPE;
V_CLIENT					  CLIENT.ID_CLIENT%TYPE;
V_DESCRIPCIO_SERVEI	SERVEI_WEB.DESCRIPCIO%TYPE;
V_NOM_REGLA				  REGLA.NOM%TYPE;

V_PARAM_ENTRADA     VARCHAR2(500);
V_ERR_TXT           VARCHAR2(200);
V_CAMP_NULL         VARCHAR2(20);
V_NOM_CLIENT			  VARCHAR2(100);
V_NOM_CLIENT_COMP	  VARCHAR2(300);
V_COGNOM_1_CLIENT	  VARCHAR2(50);
V_COGNOM_2_CLIENT	  VARCHAR2(50);
RSP	                VARCHAR2(500);
V_NOM_PKG           VARCHAR2(50):= 'PKG_FUNCIONS.FU_SERVEIS_NO_DISP_MES';

V_DIA               NUMBER;
V_REG_NO_DISP_DIA   NUMBER;
V_TEMPS_DISP_MES    FLOAT;
V_PERCENT_DISP_MES  FLOAT;
i                   INTEGER:=1;

V_RESULTAT          TA_SERVEIS_NO_DISP_MES := TA_SERVEIS_NO_DISP_MES();
EXC_CAMP_NULL       EXCEPTION;

type R_CURSOR IS REF CURSOR;
CUR R_CURSOR;

BEGIN
   
   V_PARAM_ENTRADA:= 'P_ANY: '||P_ANY||', P_MES: '||P_MES;
                 
  --verifica que els camps no siguin nuls
   IF (P_ANY IS NULL) THEN 
     V_CAMP_NULL:= 'P_ANY';
     RAISE EXC_CAMP_NULL;
   END IF;
   
   IF (P_MES IS NULL) THEN 
     V_CAMP_NULL:= 'P_MES';
     RAISE EXC_CAMP_NULL;     
   END IF;  
   
   OPEN CUR FOR   SELECT REGISTRE.SERVEI_WEB, 
                         SERVEI_WEB.DESCRIPCIO,
                         CLIENT.ID_CLIENT,
                         CLIENT.NOM,
                         CLIENT.PRIMER_COGNOM,
                         CLIENT.SEGON_COGNOM,
								         REGISTRE.REGLA,
                         REGLA.NOM,
								         EXTRACT(DAY FROM REGISTRE.DATA_INICI),
								         COUNT(*)
								    FROM REGISTRE,
                         CLIENT,
                         SERVEI_WEB,
                         REGLA
                   WHERE REGLA.ID_REGLA = REGISTRE.REGLA
                     AND SERVEI_WEB.ID_SERVEI_WEB = REGISTRE.SERVEI_WEB
                     AND CLIENT.ID_CLIENT = SERVEI_WEB.CLIENT
                     AND EXTRACT(YEAR FROM REGISTRE.DATA_INICI) = P_ANY
                     AND EXTRACT(MONTH FROM REGISTRE.DATA_INICI) = P_MES
                     AND REGISTRE.IND_DISPONIBLE = 'N'
								GROUP BY REGISTRE.SERVEI_WEB, 
                         SERVEI_WEB.DESCRIPCIO,
                         CLIENT.ID_CLIENT,
                         CLIENT.NOM,
                         CLIENT.PRIMER_COGNOM,
                         CLIENT.SEGON_COGNOM,
								         REGISTRE.REGLA,
                         REGLA.NOM,
								         EXTRACT(DAY FROM REGISTRE.DATA_INICI) 
								ORDER BY 1,2,3,4,5,6,7,8,9;    
                               
   LOOP
     FETCH CUR INTO V_SERVEI_WEB, V_DESCRIPCIO_SERVEI, V_CLIENT, V_NOM_CLIENT,
                    V_COGNOM_1_CLIENT, V_COGNOM_2_CLIENT, V_REGLA, V_NOM_REGLA, 
                    V_DIA, V_REG_NO_DISP_DIA;
     EXIT WHEN CUR%NOTFOUND;
     
       -- Cada cop que cambiem de servei web, realitzem els calculs del % i 
       -- temps estimat de disponibilitat del servei web en el mes.
       IF (V_SERVEI_WEB <> V_SERVEI_WEB_ANT)  THEN
          -- Calcul del % de disponibilitat del servei web en el mes.
          V_PERCENT_DISP_MES:= 0;
          V_PERCENT_DISP_MES:= FU_PERCENT_DISP_SERVEI_MES(V_SERVEI_WEB, P_ANY, P_MES);
          
          -- Calcul del temps estimat de disponibilitat del servei web en el mes.
          V_TEMPS_DISP_MES:= 0;
          V_TEMPS_DISP_MES:= FU_TEMPS_DISP_SERVEI_MES(V_SERVEI_WEB, P_ANY, P_MES);
          
          -- Construim el nombre del client en un sol camp.
          V_NOM_CLIENT_COMP := V_NOM_CLIENT;
          IF (TRIM(V_COGNOM_1_CLIENT) IS NOT NULL)  THEN
              V_NOM_CLIENT_COMP := TRIM(V_NOM_CLIENT_COMP) || ' ' || TRIM(V_COGNOM_1_CLIENT);
          END IF;
          IF (TRIM(V_COGNOM_2_CLIENT) IS NOT NULL)  THEN
              V_NOM_CLIENT_COMP := TRIM(V_NOM_CLIENT_COMP) || ' ' || TRIM(V_COGNOM_2_CLIENT);
          END IF;
          
          -- Inicialitzem la variable on guardem el servei web processat.
          V_SERVEI_WEB_ANT := V_SERVEI_WEB;
       END IF;
          
       -- Composem el registre de la taula amb les dades obtingudes.
       V_RESULTAT.extend;
       V_RESULTAT(i):= TY_SERVEIS_NO_DISP_MES(V_SERVEI_WEB, V_DESCRIPCIO_SERVEI, V_CLIENT, 
                       V_NOM_CLIENT_COMP, V_REGLA, V_NOM_REGLA, V_DIA, V_REG_NO_DISP_DIA, 
                       V_PERCENT_DISP_MES, V_TEMPS_DISP_MES);
       i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  -- finalitza amb xit
   RSP:= 'OK';
   SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

  --retorna resultat;
   RETURN V_RESULTAT;
   
EXCEPTION
   WHEN EXC_CAMP_NULL THEN
        RSP:= 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul'; 
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        return null;

   WHEN OTHERS THEN
        V_ERR_TXT := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR: ' || V_ERR_TXT;
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        return null;
END FU_SERVEIS_NO_DISP_MES;

/******************************************************************************
   Nom:                 FU_TEMPS_DISP_SERVEI_MES
   Propsit:            Funcio que retorna el temps estimat de disponibilitat 
                        d'un servei web, en l'any i mes que venen com a parametres.
                   
   Parmetres entrada:  -	P_SERVEI_WEB  NUMBER: identificador del servei web.
                        -	P_ANY  NUMBER: any en que es vol fer la consulta.
                        -	P_MES  NUMBER: mes en que es vol fer la consulta. 

   Parmetres sortida:  -	FLOAT 
   Retorna:             -	Retorna el temps estimat de disponibilitat, en hores,
                          del servei web.       
******************************************************************************/
FUNCTION FU_TEMPS_DISP_SERVEI_MES   (P_SERVEI_WEB IN  NUMBER,
                                     P_ANY        IN  NUMBER,
                                     P_MES        IN  NUMBER)
RETURN FLOAT
IS

V_REGLA                REGLA.ID_REGLA%TYPE;
V_PERIODICITAT         REGLA.PERIODICITAT%TYPE;
V_REGISTRES_DISP_WS    DISPONIBILITAT_MES.REGISTRES_DISP%TYPE;
V_TEMPS_DISP_WS        FLOAT := 0;
V_TEMPS_DISP_TOT_WS    FLOAT := 0;

type r_cursor IS REF CURSOR;
CUR r_cursor;

BEGIN
   
   OPEN CUR FOR SELECT  REGLA.ID_REGLA,
                        PERIODICITAT,
                        REGISTRES_DISP
                  FROM	DISPONIBILITAT_MES,
                        REGLA
                  WHERE	REGLA.ID_REGLA                 = DISPONIBILITAT_MES.REGLA
                  AND   DISPONIBILITAT_MES.ANYO				 = P_ANY
                  AND		DISPONIBILITAT_MES.MES				 = P_MES
                  AND		DISPONIBILITAT_MES.SERVEI_WEB  = P_SERVEI_WEB;
                  
   LOOP
     FETCH CUR INTO V_REGLA, V_PERIODICITAT, V_REGISTRES_DISP_WS;
     EXIT WHEN CUR%NOTFOUND;                  
    
        -- Calculem el temps estimat de disponibilitat de la regla en
        -- el mes, en segons, a partir dels registres de disponibilitat 
        -- i la periodicitat d'execucio de la regla.
        IF (V_REGISTRES_DISP_WS > 0) THEN
          IF V_PERIODICITAT	> 0	THEN
            V_TEMPS_DISP_WS := V_REGISTRES_DISP_WS * V_PERIODICITAT;
          END IF;
        END IF;
          
        IF (V_TEMPS_DISP_WS = NULL)  THEN
          V_TEMPS_DISP_WS := 0;
        END IF;
        
        -- Acumulem el tems de cadascuna de les regles.
        V_TEMPS_DISP_TOT_WS := V_TEMPS_DISP_TOT_WS + V_TEMPS_DISP_WS;
   END LOOP;  
   CLOSE CUR;

  -- Pasem el temps de disponibilitat de segons a hores.
  IF (V_TEMPS_DISP_TOT_WS > 0) THEN
      V_TEMPS_DISP_TOT_WS := ROUND(V_TEMPS_DISP_TOT_WS / 3600, 2);
  END IF;
   
   RETURN V_TEMPS_DISP_TOT_WS;
   
END FU_TEMPS_DISP_SERVEI_MES;

/******************************************************************************
   Nom:                 FU_PERCENT_DISP_SERVEI_MES
   Propsit:            Funcio que retorna, el % de disponibilitat d'un servei
                        web en l'any i mes que venen com a parametres.
                   
   Parmetres entrada:  -	P_SERVEI_WEB  NUMBER: identificador del servei web.
                        -	P_ANY  NUMBER: any en que es vol fer la consulta.
                        -	P_MES  NUMBER: mes en que es vol fer la consulta. 

   Parmetres sortida:  -	FLOAT 
   Retorna:             -	Retorna el % de disponibilitat del servei web.       
******************************************************************************/
FUNCTION FU_PERCENT_DISP_SERVEI_MES (P_SERVEI_WEB IN  NUMBER,
                                     P_ANY        IN  NUMBER,
                                     P_MES        IN  NUMBER)
RETURN FLOAT
IS

V_REGISTRES_TOT_WS     FLOAT;
V_REGISTRES_DISP_WS    FLOAT;
V_PERCENT_DISP_WS      FLOAT := 0;

BEGIN
   
    SELECT SUM(REGISTRES_TOT),
           SUM(REGISTRES_DISP)
    INTO	V_REGISTRES_TOT_WS,
          V_REGISTRES_DISP_WS
    FROM	DISPONIBILITAT_MES
    WHERE	ANYO				= P_ANY
    AND		MES					= P_MES
    AND		SERVEI_WEB	= P_SERVEI_WEB;
    
    IF (V_REGISTRES_TOT_WS > 0) THEN
      IF V_REGISTRES_DISP_WS	> 0	THEN
        V_PERCENT_DISP_WS := ROUND((V_REGISTRES_DISP_WS / V_REGISTRES_TOT_WS) * 100, 3);
      END IF;
    END IF;
      
    IF (V_PERCENT_DISP_WS = NULL)  THEN
      V_PERCENT_DISP_WS := 0;
      END IF;
      
   RETURN V_PERCENT_DISP_WS;
   
END FU_PERCENT_DISP_SERVEI_MES;

/******************************************************************************
   Nom:                 FU_DB_GETCURRVAL
   Propsit:            Funcio que retorna, donat un nom de taula que rep com parametre
											  d'entrada, l'identificador autoincremental de la fila que s'acava 
											  de donar d'alta, com a consequencia de la crida a un procediment d'alta.
                   
   Parmetres entrada:  -	P_NOM_TAULA  VARCHAR2: nom de la taula. 

   Parmetres sortida:  -	NUMBER 
   Retorna:             -	Retorna l'identificador de la fila creada a la taula.       
******************************************************************************/
FUNCTION FU_DB_GETCURRVAL          (P_NOM_TAULA  VARCHAR2)
RETURN  NUMBER
IS

V_CODI  NUMBER := 0;

BEGIN

   EXECUTE IMMEDIATE 'SELECT ' || P_NOM_TAULA || '_SEQ.CURRVAL FROM DUAL' INTO V_CODI;

   RETURN V_CODI;
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
     WHEN OTHERS THEN
       RAISE;
END FU_DB_GETCURRVAL;
          
END PKG_FUNCIONS;
/
spool off;
