/*
* Creaci del package PKG_CONSULTES.
* S'ha d'executar connectat com a monitor/monitor1606
*/

spool 10_PackageConsultes.log;

CREATE OR REPLACE PACKAGE PKG_CONSULTES AS

  -- Procediment que mostra per pantalla la llista dels serveis web que han tingut 
  -- registres de monitoritzaci en els quals s'ha detectat no disponibilitat en 
  -- el servei. 
  PROCEDURE SP_SERVEIS_NO_DISP_MES  (P_ANY         	  IN NUMBER,
                                     P_MES         	  IN NUMBER,
                                     RSP           		OUT VARCHAR2);

  -- Procediment que mostra per pantalla la llista de les regles de monitoritzacio
  -- que hi havia actives a la data que es pasa com a parametre, incloent les dades
  -- del servei associat. Tot ordenat per client. 
  PROCEDURE SP_REGLES_ACTIVES			  (P_DATA         	IN DATE, 
                                     RSP           		OUT VARCHAR2);

	-- Procediment que mostra per pantalla la llista de serveis web no disponibles en 
	-- aquest moment, indicant la regla i el pas en el qual esta fallant, i el temps 
	-- que porta no disponible.
  PROCEDURE SP_SERVEIS_NO_DISP      (RSP           		OUT VARCHAR2);

END PKG_CONSULTES;
/


CREATE OR REPLACE PACKAGE BODY PKG_CONSULTES AS

  /******************************************************************************
     Nom:                 SP_SERVEIS_NO_DISP_MES
     Propsit:            Procediment que mostra per pantalla la llista dels serveis 
     											web que han tingut registres de monitoritzaci en els
                          quals s'ha detectat no disponibilitat en el servei. 
     											La informaci es mostrar agrupada per dia i ordenada 
                          pel temps estimat de disponibilitat de forma descendent.  
                     
     Parmetres entrada:  -	P_ANY 	NUMBER: any en el que es vol realitzar la consulta.
                          -	P_MES 	NUMBER: mes en el que es vol realitzar la consulta.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_SERVEIS_NO_DISP_MES  (P_ANY         	  IN NUMBER,
                                     P_MES         	  IN NUMBER,
                                     RSP           		OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    
    V_CLIENT					        CLIENT.ID_CLIENT%TYPE;
    V_SERVEI_WEB					    SERVEI_WEB.ID_SERVEI_WEB%TYPE;
    V_DESCRIPCIO_SERVEI				SERVEI_WEB.DESCRIPCIO%TYPE;
    V_REGLA					        	REGLA.ID_REGLA%TYPE;
    V_NOM_REGLA				        REGLA.NOM%TYPE;
    
    V_ERR_NUM       				  NUMBER;
    V_DIA                     NUMBER;
    V_REG_NO_DISP_DIA         NUMBER;
    V_TEMPS_DISP_SERVEI_MES   FLOAT;
    V_PERCENT_DISP_SERVEI_MES FLOAT; 
   
    V_NOM_CLIENT					    VARCHAR2(300);
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_CONSULTES.SP_SERVEIS_NO_DISP_MES';
    
    -- Cursor per obtenir L'estadistica de no disponibilitat, agrupada per dia, de l'any-mes.
    -- Ordenem els resultats per temps estimat de disponibiliat, descendentment.
     CURSOR C_SERVEIS_NO_DISP_MES IS
        SELECT * FROM TABLE(PKG_FUNCIONS.FU_SERVEIS_NO_DISP_MES(P_ANY, P_MES))
        ORDER BY TEMPS_DISP_SERVEI_MES DESC, DIA ASC, SERVEI_WEB ASC, REGLA ASC; 
    
    BEGIN 
    
	     V_PARAM_ENTRADA := 	'P_ANY '||P_ANY||', P_MES '||P_MES;      
	
	      -- Verifiquem que tots els camps obligatoris vinguin informats.
	     IF (P_ANY IS NULL)  THEN
	         V_CAMP_NULL := 'P_ANY';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	      
	     IF (P_MES IS NULL)  THEN
	         V_CAMP_NULL := 'P_MES';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	      
	      -- Cabelera dels resultats.
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.PUT_LINE('SERVEIS WEB NO DISPONIBLES: ANY: '|| P_ANY ||', MES: '||P_MES);
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.NEW_LINE();
                 
			 OPEN  C_SERVEIS_NO_DISP_MES;
		                               
		   LOOP
		     FETCH C_SERVEIS_NO_DISP_MES INTO V_SERVEI_WEB, V_DESCRIPCIO_SERVEI, V_CLIENT, 
                                          V_NOM_CLIENT, V_REGLA, V_NOM_REGLA, V_DIA, V_REG_NO_DISP_DIA, 
                                          V_PERCENT_DISP_SERVEI_MES, V_TEMPS_DISP_SERVEI_MES;
		     EXIT WHEN C_SERVEIS_NO_DISP_MES%NOTFOUND;
		     
				    DBMS_OUTPUT.PUT_LINE('ID SERVEI WEB: '||V_SERVEI_WEB||
				    										 ', DESCRIPCIO SERVEI WEB: '||V_DESCRIPCIO_SERVEI||
				    										 ', ID CLIENT: '||V_CLIENT||
				    										 ', NOM CLIENT: '||V_NOM_CLIENT||
				    										 ', ID REGLA: '||V_REGLA||
				    										 ', NOM REGLA: '||V_NOM_REGLA||
				    										 ', DIA: '||V_DIA||
				    										 ', REGISTRES NO DISP. DIA: '||V_REG_NO_DISP_DIA||
				    										 ', % DISP. SERVEI WEB EN EL MES: '||V_PERCENT_DISP_SERVEI_MES||
				    										 ', TEMPS DISP. SERVEI WEB EN EL MES: '||V_TEMPS_DISP_SERVEI_MES);
            
		   END LOOP;  
		   CLOSE C_SERVEIS_NO_DISP_MES;   

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

    
   	EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
  END SP_SERVEIS_NO_DISP_MES;                                      

  /******************************************************************************
     Nom:                 SP_REGLES_ACTIVES
     Propsit:            Procediment que mostra per pantalla la llista de les regles 
     											de monitoritzacio que hi havia actives a la data que es 
     											pasa com a parametre, incloent les dades del servei associat. 
     											Tot ordenat per client.  
                     
     Parmetres entrada:  -	P_DATA 	DATE: data a la que es vol realitzar la consulta.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_REGLES_ACTIVES (P_DATA        IN DATE, 
                               RSP           OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    
    V_CLIENT					        CLIENT.ID_CLIENT%TYPE;
    V_NOM_CLIENT					    CLIENT.NOM%TYPE;
    V_PRIMER_COGNOM_CLIENT		CLIENT.PRIMER_COGNOM%TYPE;
    V_SEGON_COGNOM_CLIENT			CLIENT.SEGON_COGNOM%TYPE;
    V_SERVEI_WEB					    SERVEI_WEB.ID_SERVEI_WEB%TYPE;
    V_DESCRIPCIO_SERVEI				SERVEI_WEB.DESCRIPCIO%TYPE;
    V_TIPUS_SERVEI					  VARCHAR2(100);
    V_URL					        	  SERVEI_WEB.URL%TYPE;
    V_REGLA					        	REGLA.ID_REGLA%TYPE;
    V_NOM_REGLA				        REGLA.NOM%TYPE;
    V_ERR_NUM       				  NUMBER;
   
    V_DATA_000000		 				  DATE;
    V_DATA_235959		 				  DATE;
    
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_CONSULTES.SP_REGLES_ACTIVES';
    
    -- Cursor per obtenir les regles de monitoritzaci actives a una data.
     CURSOR C_REGLES_ACTIVES IS
			 SELECT  SERVEI_WEB.CLIENT,
			         CLIENT.NOM,   
			         CLIENT.PRIMER_COGNOM,   
			         CLIENT.SEGON_COGNOM,  
			         SERVEI_WEB.ID_SERVEI_WEB,   
			         SERVEI_WEB.DESCRIPCIO,   
			         DECODE (SERVEI_WEB.TIPUS_SERVEI, 'H', 'HTTP', 'S', 'SOAP-XML', 'R', 'REST-JSON','OTROS'),   
			         SERVEI_WEB.URL,   
			         REGLA.ID_REGLA,   
			         REGLA.NOM   
			    FROM REGLA,   
			         SERVEI_WEB,   
			         CLIENT  
			   WHERE ( SERVEI_WEB.ID_SERVEI_WEB = REGLA.SERVEI_WEB )   
			     AND ( CLIENT.ID_CLIENT = SERVEI_WEB.CLIENT )   
			     AND ( REGLA.ESTAT = 'A' )   
			     AND ( REGLA.DATA_ESTAT <= V_DATA_235959 )
			        
			   UNION  
			    
			  SELECT SERVEI_WEB.CLIENT,
			         CLIENT.NOM,   
			         CLIENT.PRIMER_COGNOM,   
			         CLIENT.SEGON_COGNOM,  
			         SERVEI_WEB.ID_SERVEI_WEB,   
			         SERVEI_WEB.DESCRIPCIO,   
			         DECODE (SERVEI_WEB.TIPUS_SERVEI, 'H', 'HTTP', 'S', 'SOAP-XML', 'R', 'REST-JSON','OTROS'),   
			         SERVEI_WEB.URL,   
			         HIST_ESTAT_REGLA.REGLA,   
			         REGLA.NOM  
			    FROM HIST_ESTAT_REGLA,   
			         SERVEI_WEB,   
			         REGLA,   
			         CLIENT  
			   WHERE ( REGLA.ID_REGLA = HIST_ESTAT_REGLA.REGLA )   
			     AND ( SERVEI_WEB.ID_SERVEI_WEB = REGLA.SERVEI_WEB )   
			     AND ( CLIENT.ID_CLIENT = SERVEI_WEB.CLIENT )   
			     AND ( HIST_ESTAT_REGLA.ESTAT = 'A' )   
			     AND ( HIST_ESTAT_REGLA.DATA_INICI <= V_DATA_235959)   
			     AND ( HIST_ESTAT_REGLA.DATA_FI >= V_DATA_000000)   
			ORDER BY 1 ASC,   
			         5 ASC,   
			         9 ASC  ; 
    
    BEGIN 
    
	     V_PARAM_ENTRADA := 	'P_DATA '||P_DATA;      
	
	      -- Verifiquem que tots els camps obligatoris vinguin informats.
	     IF (P_DATA IS NULL)  THEN
	         V_CAMP_NULL := 'P_DATA';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	      
	      -- La data d'inici es la data vinguda per parametre sense hores.
	      V_DATA_000000 := TRUNC(P_DATA);		
	      
	      -- La data fi es la data vinguda per parametre mes 1 dia, sense hores.
	      V_DATA_235959 := V_DATA_000000 + 1;

	      -- Cabelera dels resultats.
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.PUT_LINE('REGLES ACTIVES A DATA: '|| P_DATA );
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.NEW_LINE();
                 
			 OPEN  C_REGLES_ACTIVES;
		                               
		   LOOP
		     FETCH C_REGLES_ACTIVES INTO V_CLIENT, V_NOM_CLIENT, V_PRIMER_COGNOM_CLIENT, V_SEGON_COGNOM_CLIENT,
		     														 V_SERVEI_WEB, V_DESCRIPCIO_SERVEI, V_TIPUS_SERVEI, V_URL, V_REGLA,
		     														 V_NOM_REGLA;
		     EXIT WHEN C_REGLES_ACTIVES%NOTFOUND;
		     
				    DBMS_OUTPUT.PUT_LINE('ID CLIENT: '||V_CLIENT||
				    										 ', NOM CLIENT: '||V_NOM_CLIENT||
				    										 ', PRIMER COGNOM CLIENT: '||V_PRIMER_COGNOM_CLIENT||
				    										 ', SEGON COGNOM CLIENT: '||V_SEGON_COGNOM_CLIENT||
				    										 ', ID SERVEI WEB: '||V_SERVEI_WEB||
				    										 ', DESCRIPCIO SERVEI WEB: '||V_DESCRIPCIO_SERVEI||
				    										 ', TIPUS SERVEI WEB: '||V_TIPUS_SERVEI||
				    										 ', URL SERVEI WEB: '||V_URL||
				    										 ', ID REGLA: '||V_REGLA||
				    										 ', NOM REGLA: '||V_NOM_REGLA);
            DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
            
		   END LOOP;  
		   CLOSE C_REGLES_ACTIVES;   

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
       SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
   	EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;
    
  END SP_REGLES_ACTIVES; 

  /******************************************************************************
     Nom:                 SP_SERVEIS_NO_DISP
     Propsit:            Procediment que mostra per pantalla la llista de serveis
     											web no disponibles en aquest moment, indicant la regla i
     											el pas en el qual esta fallant, i el temps que porta no
     											disponible. 
                     
     Parmetres entrada:  Cap.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_SERVEIS_NO_DISP (RSP           OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    
    V_SERVEI_WEB		        	SERVEI_WEB.ID_SERVEI_WEB%TYPE;
    V_CLIENT		        			SERVEI_WEB.CLIENT%TYPE;
    V_DESCRIPCIO		        	SERVEI_WEB.DESCRIPCIO%TYPE;
    V_PRIMER_REG_ERROR        REGISTRE.ID_REGISTRE%TYPE;
    V_REGLA        						REGISTRE.REGLA%TYPE;
    V_DATA_INICI        			REGISTRE.DATA_INICI%TYPE;
    V_DATA_FI        					REGISTRE.DATA_FI%TYPE;
    V_ULTIM_PAS        				REGISTRE.ULTIM_PAS%TYPE;
    V_CODI_ERROR        			REGISTRE.CODI_ERROR%TYPE;

    V_ERR_NUM       				  NUMBER;
    V_TEMPS_SENSE_DISP			  FLOAT;
   
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_CONSULTES.SP_SERVEIS_NO_DISP';
    
    -- Cursor per obtenir els serveis web no disponibles en el moment actual.
     CURSOR C_SERVEIS_NO_DISP IS
      SELECT ID_SERVEI_WEB,
      			 CLIENT,
      			 DESCRIPCIO
        FROM SERVEI_WEB
       WHERE SERVEI_WEB.ESTAT = 'N'; 
    
    BEGIN 
    
	     V_PARAM_ENTRADA := 	'DATA DEL SISTEMA: '||SYSDATE;      
   
	      -- Cabelera dels resultats.
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.PUT_LINE('SERVEIS WEB NO DISPONIBLES');
	      DBMS_OUTPUT.PUT_LINE('DATA: '||SYSDATE);
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.NEW_LINE();
                 
			 OPEN  C_SERVEIS_NO_DISP;
		                               
		   LOOP
		     FETCH C_SERVEIS_NO_DISP INTO V_SERVEI_WEB, V_CLIENT, V_DESCRIPCIO;
		     EXIT WHEN C_SERVEIS_NO_DISP%NOTFOUND;
		     
	      		-- Obtenim el registre en el que ha comenat la no disponibilita del servei web.
	      		-- Aquest registre sera el primer que ha donat error, i que no te cap registre
	      		-- posterior amb disponibilitat.
						SELECT MIN(ID_REGISTRE)
						INTO   V_PRIMER_REG_ERROR 
						FROM REGISTRE 
						WHERE SERVEI_WEB  = V_SERVEI_WEB
						AND IND_DISPONIBLE = 'N'
						AND ID_REGISTRE > (SELECT MAX(ID_REGISTRE)
						                    FROM REGISTRE 
						                    WHERE SERVEI_WEB  = V_SERVEI_WEB
						                    AND IND_DISPONIBLE = 'S');
             
             -- Dades del registre.
             IF (V_PRIMER_REG_ERROR > 0)	THEN
								SELECT REGLA,
											 DATA_INICI,
											 DATA_FI,
											 ULTIM_PAS,
											 CODI_ERROR
								INTO   V_REGLA,
											 V_DATA_INICI, 
											 V_DATA_FI, 
											 V_ULTIM_PAS, 
											 V_CODI_ERROR 
								FROM REGISTRE 
								WHERE ID_REGISTRE  = V_PRIMER_REG_ERROR;
                
                -- Calculem el temps que el servei web porta no disponible (en segons).
                V_TEMPS_SENSE_DISP := (((SYSDATE - V_DATA_INICI) * 24) * 60) * 60;
								
								-- Mostrem les dades del registre per pantalla.
				    		DBMS_OUTPUT.PUT_LINE('SERVEI WEB: '||V_SERVEI_WEB||
                                     ', DESCRIPCIO SERVEI WEB: '||V_DESCRIPCIO||
                                     ', CLIENT: '||V_CLIENT||
                                     ', REGLA: '||V_REGLA||
                                     ', ID PRIMER REGISTRE ERRONI: '||V_PRIMER_REG_ERROR||
                                     ', DATA INICI REGISTRE: '||V_DATA_INICI||
                                     ', DATA FI REGISTRE: '||V_DATA_FI||
                                     ', ULTIM PAS EXECUTAT: '||V_ULTIM_PAS||
                                     ', CODI ERROR: '||V_CODI_ERROR||
                                     ', TEMPS NO DISPONIBLE (en segons): '||V_TEMPS_SENSE_DISP);

           	 END IF;
		   END LOOP;  
		   CLOSE C_SERVEIS_NO_DISP;   

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
   	EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
  END SP_SERVEIS_NO_DISP; 
          
END PKG_CONSULTES;
/
spool off;