/*
* Creaci del package PKG_CONSULTES_DW.
* S'ha d'executar connectat com a monitor/monitor1606
*/

spool 11_PackageConsultesDw.log;

CREATE OR REPLACE PACKAGE PKG_CONSULTES_DW AS

  -- Procediment que mostra per pantalla el nombre de registres de monitoritzacio que han
  -- donat com a resultat 'disponibilitat del servei'. 
  PROCEDURE SP_DW_DISP_SERVEI    	(P_ANY         			IN DW_RESULTAT_DISP.ANYO%TYPE, 
                                   P_MES         			IN DW_RESULTAT_DISP.MES%TYPE, 
                                   RSP           			OUT VARCHAR2);

  -- Procediment que mostra per pantalla el nombre de registres de monitoritzacio que han
  -- donat com a resultat 'el servei web no respn'. 
  PROCEDURE SP_DW_SERVEI_NO_RESP 	(P_ANY         			IN DW_RESULTAT_DISP.ANYO%TYPE, 
                                   P_MES         			IN DW_RESULTAT_DISP.MES%TYPE, 
                                   RSP           			OUT VARCHAR2);

  -- Procediment que mostra per pantalla el nombre de registres de monitoritzacio que han
  -- donat com a resultat 'la resposta del servei web no cont la cadena esperada'. 
  PROCEDURE SP_DW_SERVEI_RESP_KO 	(P_ANY         			IN DW_RESULTAT_DISP.ANYO%TYPE, 
                                   P_MES         			IN DW_RESULTAT_DISP.MES%TYPE, 
                                   RSP           			OUT VARCHAR2);

  -- Procediment que mostra per pantalla el nombre de serveis web que han tingut una  
  -- disponibilitat entre el 95% i el 100%, i els que han estat entre el 0% i el 95%. 
  PROCEDURE SP_DW_SERVEI_TRAM_DISP 	(P_ANY         		IN DW_DISP_SERVEI_TRAM.ANYO%TYPE, 
                                   	 P_MES         		IN DW_DISP_SERVEI_TRAM.MES%TYPE, 
                                   	 RSP           		OUT VARCHAR2);

  -- Procediment que mostra per pantalla el dia (o dies) del mes on menys disponibilitat
  -- de serveis hi ha hagut en global. 
  PROCEDURE SP_DW_DIA_MENYS_DISP   	(P_ANY         		IN DW_DIA_MENYS_DISP.ANYO%TYPE, 
                                   	 P_MES         		IN DW_DIA_MENYS_DISP.MES%TYPE, 
                                   	 RSP           		OUT VARCHAR2);

  -- Procediment que mostra per pantalla el client (o clients) que ha tinut menys 
  -- disponibilitat de serveis en el mes. 
  PROCEDURE SP_DW_CLIENT_MENYS_DISP	(P_ANY         		IN DW_CLIENT_MENYS_DISP.ANYO%TYPE, 
                                   	 P_MES         		IN DW_CLIENT_MENYS_DISP.MES%TYPE, 
                                   	 RSP           		OUT VARCHAR2);

  -- Procediment que mostra per pantalla l'identificador de la regla de monitoritzacio
  -- que mes temps mig triga en executar tots els seus passos. 
  PROCEDURE SP_DW_REGLA_MES_TEMPS		(P_ANY         		IN DW_REGLA_MES_TEMPS.ANYO%TYPE, 
                                   	 P_MES         		IN DW_REGLA_MES_TEMPS.MES%TYPE, 
                                   	 RSP           		OUT VARCHAR2);

END PKG_CONSULTES_DW;
/


CREATE OR REPLACE PACKAGE BODY PKG_CONSULTES_DW AS
  
  /******************************************************************************
     Nom:                 SP_DW_DISP_SERVEI
     Propsit:            Procediment que mostra per pantalla el nombre de registres de 
                          monitoritzacio que han donat com a resultat 'disponibilitat del servei'. 
                     
     Parmetres entrada:  -	P_ANY 	NUMBER(10): any al que es vol realitzar la consulta.
                          -	P_MES 	NUMBER(10): mes al que es vol realitzar la consulta.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_DW_DISP_SERVEI  (P_ANY         IN DW_RESULTAT_DISP.ANYO%TYPE, 
                                P_MES         IN DW_RESULTAT_DISP.MES%TYPE, 
                                RSP           OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    
    V_NOMBRE_REGISTRES        DW_RESULTAT_DISP.NOMBRE_REGISTRES%TYPE;
    V_ERR_NUM       				  NUMBER;
   
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_CONSULTES_DW.SP_DW_DISP_SERVEI';
    
    BEGIN 
    
	     V_PARAM_ENTRADA := 	'P_ANY '||P_ANY||', P_MES '||P_MES;      
	
	      -- Verifiquem que tots els camps obligatoris vinguin informats.
	     IF (P_ANY IS NULL)  THEN
	         V_CAMP_NULL := 'P_ANY';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	     IF (P_MES IS NULL)  THEN
	         V_CAMP_NULL := 'P_MES';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	      
	      -- Obtenim el nombre de registres amb disponibilitat, de l'any-mes.
	    SELECT NOMBRE_REGISTRES
	     INTO  V_NOMBRE_REGISTRES
	     FROM  DW_RESULTAT_DISP
	    WHERE  ANYO      			= P_ANY 
	      AND  MES       			= P_MES
	      AND  IND_DISPONIBLE = 'S';
	    
	      -- Mostrem el resultat per pantalla.
	    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
	    DBMS_OUTPUT.PUT_LINE('NOMBRE REGISTRES DE MONITORITZACI AMB RESULTAT: "DISPONIBILITAT DEL SERVEI"');
      DBMS_OUTPUT.PUT_LINE('ANY: '||P_ANY||chr(9)||'  MES: '|| P_MES);
	    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
	
	    DBMS_OUTPUT.PUT_LINE(V_NOMBRE_REGISTRES);

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
   	EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
  END SP_DW_DISP_SERVEI; 
  
  /******************************************************************************
     Nom:                 SP_DW_DISP_SERVEI
     Propsit:            Procediment que mostra per pantalla el nombre de registres de 
                          monitoritzacio que han donat com a resultat 'el servei web no respn'. 
                     
     Parmetres entrada:  -	P_ANY 	NUMBER(10): any al que es vol realitzar la consulta.
                          -	P_MES 	NUMBER(10): mes al que es vol realitzar la consulta.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_DW_SERVEI_NO_RESP (P_ANY         IN DW_RESULTAT_DISP.ANYO%TYPE, 
                                  P_MES         IN DW_RESULTAT_DISP.MES%TYPE, 
                                  RSP           OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    
    V_NOMBRE_REGISTRES        DW_RESULTAT_DISP.NOMBRE_REGISTRES%TYPE;
    V_ERR_NUM       				  NUMBER;
   
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_CONSULTES_DW.SP_DW_SERVEI_NO_RESP';
    
    BEGIN 
    
	     V_PARAM_ENTRADA := 	'P_ANY '||P_ANY||', P_MES '||P_MES;      
	
	      -- Verifiquem que tots els camps obligatoris vinguin informats.
	     IF (P_ANY IS NULL)  THEN
	         V_CAMP_NULL := 'P_ANY';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	     IF (P_MES IS NULL)  THEN
	         V_CAMP_NULL := 'P_MES';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	      
	      -- Obtenim el nombre de registres amb disponibilitat, de l'any-mes.
	    SELECT NOMBRE_REGISTRES
	     INTO  V_NOMBRE_REGISTRES
	     FROM  DW_RESULTAT_DISP
	    WHERE  ANYO      			= P_ANY 
	      AND  MES       			= P_MES
	      AND  IND_DISPONIBLE = 'N'
	      AND  CODI_ERROR 		= '001';
	    
	      -- Mostrem el resultat per pantalla.
	    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
	    DBMS_OUTPUT.PUT_LINE('NOMBRE REGISTRES DE MONITORITZACI AMB RESULTAT: "EL SERVEI WEB NO RESPN"');
      DBMS_OUTPUT.PUT_LINE('ANY: '||P_ANY||chr(9)||'  MES: '|| P_MES);
	    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
	
	    DBMS_OUTPUT.PUT_LINE(V_NOMBRE_REGISTRES);

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
   	EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
  END SP_DW_SERVEI_NO_RESP; 
  
  /******************************************************************************
     Nom:                 SP_DW_SERVEI_RESP_KO
     Propsit:            Procediment que mostra per pantalla el nombre de registres de 
                          monitoritzacio que han donat com a resultat 'la resposta del 
                          servei web no cont la cadena esperada'. 
                     
     Parmetres entrada:  -	P_ANY 	NUMBER(10): any al que es vol realitzar la consulta.
                          -	P_MES 	NUMBER(10): mes al que es vol realitzar la consulta.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_DW_SERVEI_RESP_KO 	(P_ANY         IN DW_RESULTAT_DISP.ANYO%TYPE, 
                                   P_MES         IN DW_RESULTAT_DISP.MES%TYPE, 
                                   RSP           OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    
    V_NOMBRE_REGISTRES        DW_RESULTAT_DISP.NOMBRE_REGISTRES%TYPE;
    V_ERR_NUM       				  NUMBER;
   
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_CONSULTES_DW.SP_DW_SERVEI_RESP_KO';
    
    BEGIN 
    
	     V_PARAM_ENTRADA := 	'P_ANY '||P_ANY||', P_MES '||P_MES;      
	
	      -- Verifiquem que tots els camps obligatoris vinguin informats.
	     IF (P_ANY IS NULL)  THEN
	         V_CAMP_NULL := 'P_ANY';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	     IF (P_MES IS NULL)  THEN
	         V_CAMP_NULL := 'P_MES';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	      
	      -- Obtenim el nombre de registres amb disponibilitat, de l'any-mes.
	    SELECT NOMBRE_REGISTRES
	     INTO  V_NOMBRE_REGISTRES
	     FROM  DW_RESULTAT_DISP
	    WHERE  ANYO      			= P_ANY 
	      AND  MES       			= P_MES
	      AND  IND_DISPONIBLE = 'N'
	      AND  CODI_ERROR 		= '002';
	    
	      -- Mostrem el resultat per pantalla.
	    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
	    DBMS_OUTPUT.PUT_LINE('NOMBRE REGISTRES DE MONITORITZACI AMB RESULTAT: "LA RESPOSTA DEL 
                            SERVEI WEB NO CONT LA CADENA ESPERADA"');
      DBMS_OUTPUT.PUT_LINE('ANY: '||P_ANY||chr(9)||'  MES: '|| P_MES);
	    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
	
	    DBMS_OUTPUT.PUT_LINE(V_NOMBRE_REGISTRES);

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
   	EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
  END SP_DW_SERVEI_RESP_KO; 
  
  /******************************************************************************
     Nom:                 SP_DW_SERVEI_TRAM_DISP
     Propsit:            Procediment que mostra per pantalla el nombre de serveis web que 
  												han tingut una disponibilitat entre el 95% i el 100%, i els que 
  												han estat entre el 0% i el 95%. 
                     
     Parmetres entrada:  -	P_ANY 	NUMBER(10): any al que es vol realitzar la consulta.
                          -	P_MES 	NUMBER(10): mes al que es vol realitzar la consulta.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_DW_SERVEI_TRAM_DISP 	(P_ANY         IN DW_DISP_SERVEI_TRAM.ANYO%TYPE, 
                                   	 P_MES         IN DW_DISP_SERVEI_TRAM.MES%TYPE, 
                                   	 RSP           OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    
    V_SERVEIS_0_95		        DW_DISP_SERVEI_TRAM.NOMBRE_SERVEIS%TYPE;
    V_SERVEIS_95_100	        DW_DISP_SERVEI_TRAM.NOMBRE_SERVEIS%TYPE;
    V_ERR_NUM       				  NUMBER;
   
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_CONSULTES_DW.SP_DW_SERVEI_TRAM_DISP';
    
    BEGIN 
    
	     V_PARAM_ENTRADA := 	'P_ANY '||P_ANY||', P_MES '||P_MES;      
	
	      -- Verifiquem que tots els camps obligatoris vinguin informats.
	     IF (P_ANY IS NULL)  THEN
	         V_CAMP_NULL := 'P_ANY';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	     IF (P_MES IS NULL)  THEN
	         V_CAMP_NULL := 'P_MES';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	      
	      -- Obtenim el nombre de registres amb disponibilitat, de l'any-mes.
		  SELECT DW_DISP_SERVEI_TRAM.NOMBRE_SERVEIS  
		    INTO V_SERVEIS_0_95
		    FROM DW_DISP_SERVEI_TRAM  
		   WHERE ( DW_DISP_SERVEI_TRAM.ANYO = P_ANY ) AND  
		         ( DW_DISP_SERVEI_TRAM.MES = P_MES ) AND  
		         ( DW_DISP_SERVEI_TRAM.IND_TRAM = '1' ); 
		           
		  SELECT DW_DISP_SERVEI_TRAM.NOMBRE_SERVEIS  
		    INTO V_SERVEIS_95_100
		    FROM DW_DISP_SERVEI_TRAM  
		   WHERE ( DW_DISP_SERVEI_TRAM.ANYO = P_ANY ) AND  
		         ( DW_DISP_SERVEI_TRAM.MES = P_MES ) AND  
		         ( DW_DISP_SERVEI_TRAM.IND_TRAM = '2' );
	    
	      -- Mostrem el resultat per pantalla.
	    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
	    DBMS_OUTPUT.PUT_LINE('NOMBRE SERVEIS WEB PER TRAMS DE DISPONIBILITAT');
      DBMS_OUTPUT.PUT_LINE('ANY: '||P_ANY||chr(9)||'  MES: '|| P_MES);
	    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
	    DBMS_OUTPUT.NEW_LINE();
	
	    DBMS_OUTPUT.PUT_LINE('ENTRE EL 0% I EL 95%: '||V_SERVEIS_0_95);
	    DBMS_OUTPUT.PUT_LINE('ENTRE EL 95% I EL 100%: '||V_SERVEIS_95_100);

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
   	EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
  END SP_DW_SERVEI_TRAM_DISP; 
  
  /******************************************************************************
     Nom:                 SP_DW_DIA_MENYS_DISP
     Propsit:            Procediment que mostra per pantalla el dia (o dies) del 
     											mes on menys disponibilitat de serveis hi ha hagut en global 
                     
     Parmetres entrada:  -	P_ANY 	NUMBER(10): any al que es vol realitzar la consulta.
                          -	P_MES 	NUMBER(10): mes al que es vol realitzar la consulta.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_DW_DIA_MENYS_DISP 	(P_ANY         IN DW_DIA_MENYS_DISP.ANYO%TYPE, 
                                   P_MES         IN DW_DIA_MENYS_DISP.MES%TYPE, 
                                   RSP           OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    
    V_DIA							        DW_DIA_MENYS_DISP.DIA%TYPE;
    V_PERCENTATGE_DISP        DW_DIA_MENYS_DISP.PERCENTATGE_DISP%TYPE;
    V_ERR_NUM       				  NUMBER;
   
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_CONSULTES_DW.SP_DW_DIA_MENYS_DISP';
    
    -- Cursor per obtenir el dia (o dies) amb menys disponibilitat, de l'any-mes.
     CURSOR C_DIA_MENYS_DISP IS
      SELECT DIA,
             PERCENTATGE_DISP
        FROM DW_DIA_MENYS_DISP
       WHERE DW_DIA_MENYS_DISP.ANYO = P_ANY    
         AND DW_DIA_MENYS_DISP.MES = P_MES ; 
    
    BEGIN 
    
	     V_PARAM_ENTRADA := 	'P_ANY '||P_ANY||', P_MES '||P_MES;      
	
	      -- Verifiquem que tots els camps obligatoris vinguin informats.
	     IF (P_ANY IS NULL)  THEN
	         V_CAMP_NULL := 'P_ANY';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	     IF (P_MES IS NULL)  THEN
	         V_CAMP_NULL := 'P_MES';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	      
	      -- Cabelera dels resultats.
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.PUT_LINE('DIA AMB MENYS DISPONIBILITAT DE SERVEIS');
	      DBMS_OUTPUT.PUT_LINE('ANY: '||P_ANY||chr(9)||'  MES: '|| P_MES);
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.NEW_LINE();
                 
			 OPEN  C_DIA_MENYS_DISP;
		                               
		   LOOP
		     FETCH C_DIA_MENYS_DISP INTO V_DIA, V_PERCENTATGE_DISP;
		     EXIT WHEN C_DIA_MENYS_DISP%NOTFOUND;
				    DBMS_OUTPUT.PUT_LINE('DIA: '||V_DIA||', % DISPONIBILITAT: '||V_PERCENTATGE_DISP);
		   END LOOP;  
		   CLOSE C_DIA_MENYS_DISP;   

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
   	EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
  END SP_DW_DIA_MENYS_DISP;    
  
  /******************************************************************************
     Nom:                 SP_DW_CLIENT_MENYS_DISP
     Propsit:            Procediment que mostra per pantalla el client (o clients) 
     											que ha tinut menys disponibilitat de serveis en el mes. 
                     
     Parmetres entrada:  -	P_ANY 	NUMBER(10): any al que es vol realitzar la consulta.
                          -	P_MES 	NUMBER(10): mes al que es vol realitzar la consulta.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_DW_CLIENT_MENYS_DISP  (P_ANY         IN DW_CLIENT_MENYS_DISP.ANYO%TYPE, 
                                   		P_MES         IN DW_CLIENT_MENYS_DISP.MES%TYPE, 
                                   		RSP           OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    
    V_CLIENT					        DW_CLIENT_MENYS_DISP.CLIENT%TYPE;
    V_PERCENTATGE_DISP        DW_CLIENT_MENYS_DISP.PERCENTATGE_DISP%TYPE;
    V_ERR_NUM       				  NUMBER;
   
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_CONSULTES_DW.SP_DW_CLIENT_MENYS_DISP';
    
    -- Cursor per obtenir el client (o client) amb menys disponibilitat, de l'any-mes.
     CURSOR C_CLIENT_MENYS_DISP IS
      SELECT CLIENT,
             PERCENTATGE_DISP
        FROM DW_CLIENT_MENYS_DISP
       WHERE DW_CLIENT_MENYS_DISP.ANYO = P_ANY    
         AND DW_CLIENT_MENYS_DISP.MES = P_MES ; 
    
    BEGIN 
    
	     V_PARAM_ENTRADA := 	'P_ANY '||P_ANY||', P_MES '||P_MES;      
	
	      -- Verifiquem que tots els camps obligatoris vinguin informats.
	     IF (P_ANY IS NULL)  THEN
	         V_CAMP_NULL := 'P_ANY';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	     IF (P_MES IS NULL)  THEN
	         V_CAMP_NULL := 'P_MES';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	      
	      -- Cabelera dels resultats.
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.PUT_LINE('CLIENT AMB MENYS DISPONIBILITAT DE SERVEIS');
	      DBMS_OUTPUT.PUT_LINE('ANY: '||P_ANY||chr(9)||'  MES: '|| P_MES);
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.NEW_LINE();
                 
			 OPEN  C_CLIENT_MENYS_DISP;
		                               
		   LOOP
		     FETCH C_CLIENT_MENYS_DISP INTO V_CLIENT, V_PERCENTATGE_DISP;
		     EXIT WHEN C_CLIENT_MENYS_DISP%NOTFOUND;
				    DBMS_OUTPUT.PUT_LINE('CLIENT: '||V_CLIENT||', % DISPONIBILITAT: '||V_PERCENTATGE_DISP);
		   END LOOP;  
		   CLOSE C_CLIENT_MENYS_DISP;   

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
   	EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
  END SP_DW_CLIENT_MENYS_DISP;     

  
  /******************************************************************************
     Nom:                 SP_DW_REGLA_MES_TEMPS
     Propsit:            Procediment que mostra per pantalla l'identificador de 
     											la regla de monitoritzacio que mes temps mig triga en 
     											executar tots els seus passos 
                     
     Parmetres entrada:  -	P_ANY 	NUMBER(10): any al que es vol realitzar la consulta.
                          -	P_MES 	NUMBER(10): mes al que es vol realitzar la consulta.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_DW_REGLA_MES_TEMPS  (P_ANY         IN DW_REGLA_MES_TEMPS.ANYO%TYPE, 
                                   	P_MES         IN DW_REGLA_MES_TEMPS.MES%TYPE, 
                                   	RSP           OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    
    V_REGLA					        	DW_REGLA_MES_TEMPS.REGLA%TYPE;
    V_TEMPS_MIG				        DW_REGLA_MES_TEMPS.TEMPS_MIG%TYPE;
    V_ERR_NUM       				  NUMBER;
   
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_CONSULTES_DW.SP_DW_REGLA_MES_TEMPS';
    
    -- Cursor per obtenir la regla (o regles) que triguen mes en executar-se, en l'any-mes.
     CURSOR C_REGLA_MES_TEMPS IS
      SELECT REGLA,
             TEMPS_MIG
        FROM DW_REGLA_MES_TEMPS
       WHERE DW_REGLA_MES_TEMPS.ANYO = P_ANY    
         AND DW_REGLA_MES_TEMPS.MES = P_MES ; 
    
    BEGIN 
    
	     V_PARAM_ENTRADA := 	'P_ANY '||P_ANY||', P_MES '||P_MES;      
	
	      -- Verifiquem que tots els camps obligatoris vinguin informats.
	     IF (P_ANY IS NULL)  THEN
	         V_CAMP_NULL := 'P_ANY';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	     IF (P_MES IS NULL)  THEN
	         V_CAMP_NULL := 'P_MES';
	         RAISE EXC_CAMP_NULL;
	      END IF;
	      
	      -- Cabelera dels resultats.
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.PUT_LINE('REGLA QUE MS TEMPS TRIGA EN EXECUTAR-SE');
	      DBMS_OUTPUT.PUT_LINE('ANY: '||P_ANY||chr(9)||'  MES: '|| P_MES);
		    DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
		    DBMS_OUTPUT.NEW_LINE();
                 
			 OPEN  C_REGLA_MES_TEMPS;
		                               
		   LOOP
		     FETCH C_REGLA_MES_TEMPS INTO V_REGLA, V_TEMPS_MIG;
		     EXIT WHEN C_REGLA_MES_TEMPS%NOTFOUND;
				    DBMS_OUTPUT.PUT_LINE('REGLA: '||V_REGLA||', TEMPS MIG: '||V_TEMPS_MIG);
		   END LOOP;  
		   CLOSE C_REGLA_MES_TEMPS;   

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
   	EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
  END SP_DW_REGLA_MES_TEMPS; 
          
END PKG_CONSULTES_DW;
/

spool off;