/*
* Creaci del package PKG_TAULES_DW.
* S'ha d'executar connectat com a monitor/monitor1606
*/

spool 12_PackageTaulesDw.log;

CREATE OR REPLACE PACKAGE PKG_TAULES_DW AS

  -- Procediment per donar d'alta o modificar estadistica de la taula DISPONIBILITAT_MES. 
  PROCEDURE SP_DISPONIBILITAT_MES 	(P_ID_REGISTRE      IN REGISTRE.ID_REGISTRE%TYPE,
                                     RSP               	OUT VARCHAR2);
                                 
  -- Procediment per donar d'alta o modificar estadistica de la taula DW_DISP_SERVEI_TRAM. 
  PROCEDURE SP_DW_DISP_SERVEI_TRAM (P_ANY         			IN DW_DISP_SERVEI_TRAM.ANYO%TYPE, 
                                    P_MES         			IN DW_DISP_SERVEI_TRAM.MES%TYPE, 
                                    P_IND_TRAM    			IN DW_DISP_SERVEI_TRAM.IND_TRAM%TYPE, 
                                    P_SUMA_RESTA  			IN CHAR, 
                                    RSP           			OUT VARCHAR2);

  -- Procediment per donar d'alta o modificar estadistica de la taula DISPONIBILITAT_DIA. 
  PROCEDURE SP_DISPONIBILITAT_DIA 	(P_ID_REGISTRE      IN REGISTRE.ID_REGISTRE%TYPE,
                        					 	 RSP               	OUT VARCHAR2);

  -- Procediment per donar d'alta o modificar estadistica de la taula DW_DIA_MENYS_DISP. 
  PROCEDURE SP_DW_DIA_MENYS_DISP 		(P_ANY         			IN DW_DIA_MENYS_DISP.ANYO%TYPE, 
                                  	 P_MES         			IN DW_DIA_MENYS_DISP.MES%TYPE, 
                                  	 P_DIA         			IN DW_DIA_MENYS_DISP.DIA%TYPE, 
                                  	 P_PERCENTATGE_DISP IN DW_DIA_MENYS_DISP.PERCENTATGE_DISP%TYPE, 
                                  	 RSP           			OUT VARCHAR2);

  -- Procediment per donar d'alta o modificar estadistica de la taula DW_CLIENT_MENYS_DISP. 
  PROCEDURE SP_DW_CLIENT_MENYS_DISP (P_ID_REGISTRE      IN REGISTRE.ID_REGISTRE%TYPE, 
                                  	 RSP           			OUT VARCHAR2);

  -- Procediment per donar d'alta o modificar estadistica de la taula DW_REGLA_MES_TEMPS. 
  PROCEDURE SP_DW_REGLA_MES_TEMPS   (P_ID_REGISTRE      IN REGISTRE.ID_REGISTRE%TYPE, 
                                  	 RSP           			OUT VARCHAR2);


END PKG_TAULES_DW;
/


CREATE OR REPLACE PACKAGE BODY PKG_TAULES_DW AS

   
  /******************************************************************************
     Nom:                 SP_DISPONIBILITAT_MES
     Propsit:            Actualitza la taula DISPONIBILITAT_MES. 
                     
     Parmetres entrada:  -	P_ID_REGISTRE NUMBER(10): identificaci del registre.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_DISPONIBILITAT_MES (P_ID_REGISTRE   IN REGISTRE.ID_REGISTRE%TYPE,
                                    RSP             OUT VARCHAR2)                        		 
    IS

   EXC_CAMP_NULL         		EXCEPTION;
   EXC_REGISTRE_NOEXIST			EXCEPTION;
   EXC_PAS_NOEXIST					EXCEPTION;

	 V_ANY						    		NUMBER;
	 V_MES						    		NUMBER;
	 V_COUNT					    		NUMBER := 0;
	 V_COUNT_DISP_WS     			NUMBER := 0;
	 V_REG_DISP				    		NUMBER := 0;
	 V_REG_NO_DISP		    		NUMBER := 0;
	 V_TEMPS_RESP_DISP				NUMBER := 0;   
   V_ERR_NUM       					NUMBER;
   V_TEMPS_RESPOSTA_TOT			NUMBER;
   V_REGISTRES_TOT_WS      	NUMBER := 0;
   V_REGISTRES_DISP_WS     	NUMBER := 0;
   
	 V_PERCENT_DISP		    		FLOAT  := 0;
   V_PERCENT_DISP_WS_ANT   	FLOAT  := 0;
   V_PERCENT_DISP_WS_ACT   	FLOAT  := 0;
   
   V_IND_TRAM_ANT          	CHAR(1);
   V_IND_TRAM_ACT          	CHAR(1);
   
   V_PARAM_ENTRADA 					VARCHAR2(500);
   V_ERR_TXT       					VARCHAR2(100);
   V_CAMP_NULL      				VARCHAR2(20);
   V_NOM_PKG             		VARCHAR2(50):= 'PKG_TAULES_DW.SP_DISPONIBILITAT_MES';
   
	 V_CANVI_DE_TRAM			    BOOLEAN := FALSE;
   
   V_REGISTRE								REGISTRE%ROWTYPE;
	 V_DISPONIBILITAT_MES			DISPONIBILITAT_MES%ROWTYPE;

   
   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_REGISTRE '||P_ID_REGISTRE;      

     -- Verifiquem que el registre existeixi.
     IF (P_ID_REGISTRE IS NULL)  THEN
         V_CAMP_NULL := 'ID_REGISTRE';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     -- Obtenim les dades del registre.
       SELECT *
         INTO V_REGISTRE				
         FROM REGISTRE
        WHERE ID_REGISTRE = P_ID_REGISTRE;

  		V_ANY := EXTRACT(YEAR FROM V_REGISTRE.DATA_INICI);
  		V_MES := EXTRACT(MONTH FROM V_REGISTRE.DATA_INICI);
  		
  		IF V_REGISTRE.IND_DISPONIBLE = 'S'	THEN
  				V_REG_DISP 		    := 1;
  				V_PERCENT_DISP     := 100;
          V_TEMPS_RESP_DISP  := V_REGISTRE.TEMPS_RESPOSTA_TOT;
  		ELSE
  				V_REG_NO_DISP 	    := 1;
  		END IF;    
  		

       -- Calculem el % de disponibilitat del Servei Web, abans
       -- de modificar la taula DISPONIBILITAT_MES.
       -- Aixo ens servira per saber si les dades del registre 
       -- provoquen un canvi de tram de disponibilitat.
       
 			SELECT COUNT(*)
  		INTO	V_COUNT_DISP_WS
  		FROM	DISPONIBILITAT_MES
  		WHERE	ANYO				= V_ANY
  		AND		MES					= V_MES
  		AND		SERVEI_WEB	= V_REGISTRE.SERVEI_WEB;
      
      V_IND_TRAM_ANT := '0';
       
      IF (V_COUNT_DISP_WS > 0) THEN
      
          SELECT SUM(REGISTRES_TOT),
                 SUM(REGISTRES_DISP)
          INTO	V_REGISTRES_TOT_WS,
                V_REGISTRES_DISP_WS
          FROM	DISPONIBILITAT_MES
          WHERE	ANYO				= V_ANY
          AND		MES					= V_MES
          AND		SERVEI_WEB	= V_REGISTRE.SERVEI_WEB;
          
          IF (V_REGISTRES_TOT_WS > 0) THEN
            IF V_REGISTRES_DISP_WS	> 0	THEN
              V_PERCENT_DISP_WS_ANT := ROUND((V_REGISTRES_DISP_WS / V_REGISTRES_TOT_WS) * 100, 3);
            END IF;
          END IF;
          
         -- Segons el % de disponibilitat, assignem un tram o un altre.
          IF V_PERCENT_DISP_WS_ANT <= 95.00	THEN
            V_IND_TRAM_ANT := '1';
          ELSE
            V_IND_TRAM_ANT := '2';
          END IF;          
      END IF;      
      
     ----------------------------------------------------------------------------------------
		 -- ACTUALITZACIO DE LA TAULA DISPONIBILITAT_MES.
     ----------------------------------------------------------------------------------------

      -- Si no existeix registre a DISPONIBILITAT_MES per la regla i servei, el donem
      -- d'alta. Si ja existeix, la modifiquem considerant les noves dades.
       
 			SELECT COUNT(*)
  		INTO	V_COUNT
  		FROM	DISPONIBILITAT_MES
  		WHERE	ANYO				= V_ANY
  		AND		MES					= V_MES
  		AND		SERVEI_WEB	= V_REGISTRE.SERVEI_WEB
  		AND		REGLA				= V_REGISTRE.REGLA;

  		IF (V_COUNT = 1) THEN
      
        SELECT *
        INTO	V_DISPONIBILITAT_MES
        FROM	DISPONIBILITAT_MES
        WHERE	ANYO				= V_ANY
        AND		MES					= V_MES
        AND		SERVEI_WEB	= V_REGISTRE.SERVEI_WEB
        AND		REGLA				= V_REGISTRE.REGLA;
        
        V_DISPONIBILITAT_MES.REGISTRES_TOT 			:= V_DISPONIBILITAT_MES.REGISTRES_TOT 			+ 1;
        V_DISPONIBILITAT_MES.REGISTRES_DISP 		:= V_DISPONIBILITAT_MES.REGISTRES_DISP 		  + V_REG_DISP;
        V_DISPONIBILITAT_MES.REGISTRES_NO_DISP 	:= V_DISPONIBILITAT_MES.REGISTRES_NO_DISP 	+ V_REG_NO_DISP;
        
        IF V_DISPONIBILITAT_MES.REGISTRES_DISP	> 0	THEN
        	V_DISPONIBILITAT_MES.PERCENTATGE_DISP := ROUND((V_DISPONIBILITAT_MES.REGISTRES_DISP / V_DISPONIBILITAT_MES.REGISTRES_TOT) * 100, 3);
        END IF;
        
        V_DISPONIBILITAT_MES.TEMPS_EXECUCIO_TOT 	:= V_DISPONIBILITAT_MES.TEMPS_EXECUCIO_TOT 	+ V_REGISTRE.TEMPS_RESPOSTA_TOT;
        V_DISPONIBILITAT_MES.TEMPS_EXECUCIO_DISP  := V_DISPONIBILITAT_MES.TEMPS_EXECUCIO_DISP	+ V_TEMPS_RESP_DISP;

        IF V_DISPONIBILITAT_MES.TEMPS_EXECUCIO_DISP	> 0	THEN
        	V_DISPONIBILITAT_MES.TEMPS_EXECUCIO_MIG_DISP := ROUND(V_DISPONIBILITAT_MES.TEMPS_EXECUCIO_DISP / V_DISPONIBILITAT_MES.REGISTRES_DISP, 0);
        END IF;
        
        UPDATE	DISPONIBILITAT_MES SET
        				ROW = V_DISPONIBILITAT_MES
         WHERE 	ID_DISPONIBILITAT_MES = V_DISPONIBILITAT_MES.ID_DISPONIBILITAT_MES;
         
  		ELSE
        
	      INSERT INTO DISPONIBILITAT_MES (ANYO, MES, DATA_CALCUL, SERVEI_WEB, REGLA, REGISTRES_TOT, REGISTRES_DISP, 
	      			REGISTRES_NO_DISP, PERCENTATGE_DISP, TEMPS_EXECUCIO_TOT, TEMPS_EXECUCIO_DISP, TEMPS_EXECUCIO_MIG_DISP)
	      VALUES(V_ANY, V_MES, SYSDATE, V_REGISTRE.SERVEI_WEB, V_REGISTRE.REGLA, 1, V_REG_DISP, V_REG_NO_DISP, 
	      			V_PERCENT_DISP, V_REGISTRE.TEMPS_RESPOSTA_TOT, V_TEMPS_RESP_DISP, V_REGISTRE.TEMPS_RESPOSTA_TOT);
  		END IF;  		 
      
     ----------------------------------------------------------------------------------------
     -- ACTUALITZEM LES DADES ESTADISTIQUES DE DW_DISP_SERVEI_TRAM.
     ----------------------------------------------------------------------------------------
     
      -- Calculem el nou % de disponibilitat del Servei Web, despres
      -- de modificar la taula DISPONIBILITAT_MES. 
       
      V_REGISTRES_TOT_WS  := V_REGISTRES_TOT_WS  + 1;
      V_REGISTRES_DISP_WS := V_REGISTRES_DISP_WS + V_REG_DISP;

      IF V_REGISTRES_DISP_WS	> 0	THEN
        V_PERCENT_DISP_WS_ACT := ROUND((V_REGISTRES_DISP_WS / V_REGISTRES_TOT_WS) * 100, 3);
      END IF;
      
      -- Segons el % de disponibilitat, assignem un tram o un altre.
      IF V_PERCENT_DISP_WS_ACT <= 95.00	THEN
        V_IND_TRAM_ACT := '1';
      ELSE
        V_IND_TRAM_ACT := '2';
      END IF;          
      
      -- Comprovem si el canvi de % de disponibilitat comporta un canvi de tram.
      IF (V_IND_TRAM_ANT <> V_IND_TRAM_ACT)	THEN
        IF (V_IND_TRAM_ANT = '0')	THEN
          -- Si el servei web no estava reflectit en l'estadistica, al ser un
          -- % de disponibilitat nou en el periode, incrementem el comptador
          -- de serveis.
          SP_DW_DISP_SERVEI_TRAM(V_ANY, V_MES, V_IND_TRAM_ACT, 'S', RSP);
        ELSE 
          -- Si el servei web canvia de tram, fem les modificacions pertinents 
          -- a la taula estadistica (restem a l'anterior i sumem al nou).
          SP_DW_DISP_SERVEI_TRAM(V_ANY, V_MES, V_IND_TRAM_ANT, 'R', RSP);
          SP_DW_DISP_SERVEI_TRAM(V_ANY, V_MES, V_IND_TRAM_ACT, 'S', RSP);
        END IF;
      END IF;

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
     SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGISTRE_NOEXIST THEN
        RSP := 'ERROR: El Registre amb identificador: ' || P_ID_REGISTRE || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

   END SP_DISPONIBILITAT_MES;
   
  /******************************************************************************
     Nom:                 SP_DW_DISP_SERVEI_TRAM
     Propsit:            Actualitza l'estadistica de la taula DW_DISP_SERVEI_TRAM. 
                     
     Parmetres entrada:  -	P_ANY 				NUMBER(10): any en el que s'ha modificat el registre.
                          -	P_MES 				NUMBER(10): mes en el que s'ha modificat el registre.
                          -	P_IND_TRAM  	CHAR			: codi del tram (1: <=95%, 2: > 95%).
                          -	P_SUMA_RESTA  CHAR			: indicador de si ha de (S)umar o (R)estar.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_DW_DISP_SERVEI_TRAM (P_ANY         IN DW_DISP_SERVEI_TRAM.ANYO%TYPE, 
                                    P_MES         IN DW_DISP_SERVEI_TRAM.MES%TYPE, 
                                    P_IND_TRAM    IN DW_DISP_SERVEI_TRAM.IND_TRAM%TYPE, 
                                    P_SUMA_RESTA  IN CHAR, 
                                    RSP           OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    EXC_TRAM_NOEXIST					EXCEPTION;
    
    V_ID_DW_DISP_SERVEI_TRAM  DW_DISP_SERVEI_TRAM.ID_DW_DISP_SERVEI_TRAM%TYPE;
    V_NOMBRE_SERVEIS          DW_DISP_SERVEI_TRAM.NOMBRE_SERVEIS%TYPE;
    
    V_COUNT                   NUMBER(10) := '0';
    V_ERR_NUM       				  NUMBER;
   
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_TAULES_DW.SP_DW_DISP_SERVEI_TRAM';
    
    BEGIN 
    
     V_PARAM_ENTRADA := 	'P_ANY '||P_ANY||', P_MES '||P_MES||', P_IND_TRAM '||P_IND_TRAM||
                          ', P_SUMA_RESTA '||P_SUMA_RESTA;      

      -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_ANY IS NULL)  THEN
         V_CAMP_NULL := 'P_ANY';
         RAISE EXC_CAMP_NULL;
      END IF;
     IF (P_MES IS NULL)  THEN
         V_CAMP_NULL := 'P_MES';
         RAISE EXC_CAMP_NULL;
      END IF;
     IF (P_IND_TRAM IS NULL)  THEN
         V_CAMP_NULL := 'P_IND_TRAM';
         RAISE EXC_CAMP_NULL;
      END IF;
     IF (P_SUMA_RESTA IS NULL)  THEN
         V_CAMP_NULL := 'P_SUMA_RESTA';
         RAISE EXC_CAMP_NULL;
      END IF;
      
      -- Mirem si existeix estadistica per l'any-mes-tram.
    SELECT COUNT(*)
     INTO  V_COUNT
     FROM  DW_DISP_SERVEI_TRAM
    WHERE  ANYO      = P_ANY 
      AND  MES       = P_MES
      AND  IND_TRAM  = P_IND_TRAM;
    
    IF V_COUNT > 0 THEN
          -- Si el tram existeix, sumem o restem.
        SELECT ID_DW_DISP_SERVEI_TRAM,
               NOMBRE_SERVEIS
         INTO  V_ID_DW_DISP_SERVEI_TRAM,
               V_NOMBRE_SERVEIS
         FROM  DW_DISP_SERVEI_TRAM
        WHERE  ANYO      = P_ANY 
          AND  MES       = P_MES
          AND  IND_TRAM  = P_IND_TRAM;
    
          IF P_SUMA_RESTA = 'S' THEN
            V_NOMBRE_SERVEIS := V_NOMBRE_SERVEIS + 1;
          ELSE
            V_NOMBRE_SERVEIS := V_NOMBRE_SERVEIS - 1;
          END IF;
          
          -- Actualitzem el tram amb el nou valor.
          UPDATE DW_DISP_SERVEI_TRAM SET
                NOMBRE_SERVEIS = V_NOMBRE_SERVEIS
          WHERE ID_DW_DISP_SERVEI_TRAM = V_ID_DW_DISP_SERVEI_TRAM;
          
    ELSE 
      -- Si el tram no existeix, si hem de sumar el donem d'alta.
      IF P_SUMA_RESTA = 'S' THEN
	      INSERT INTO DW_DISP_SERVEI_TRAM (ANYO, MES, DATA_CALCUL, IND_TRAM, NOMBRE_SERVEIS)
	           VALUES(P_ANY, P_MES, SYSDATE, P_IND_TRAM, 1);
      ELSE
        -- Si el tram no existeix i hem de restar, error.
        RAISE EXC_TRAM_NOEXIST;
      END IF;
    END IF;
    
   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_TRAM_NOEXIST THEN
        RSP := 'ERROR: El tram amb identificador: ' || P_IND_TRAM || ' no existeix per periode indicat, i per tant no es pot restar';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
  END SP_DW_DISP_SERVEI_TRAM; 
  
  /******************************************************************************
     Nom:                 SP_DISPONIBILITAT_DIA
     Propsit:            Actualitza la taula DISPONIBILITAT_DIA. 
                     
     Parmetres entrada:  -	P_ID_REGISTRE NUMBER(10): identificaci del registre.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_DISPONIBILITAT_DIA (P_ID_REGISTRE   	IN REGISTRE.ID_REGISTRE%TYPE,
                                    RSP               OUT VARCHAR2)                        		 
    IS

   EXC_CAMP_NULL         		EXCEPTION;
   EXC_REGISTRE_NOEXIST			EXCEPTION;
   EXC_PAS_NOEXIST					EXCEPTION;

	 V_ANY						    		NUMBER;
	 V_MES						    		NUMBER;
	 V_DIA						    		NUMBER;
	 V_COUNT					    		NUMBER := 0;
	 V_REG_DISP				    		NUMBER := 0;
	 V_REG_NO_DISP		    		NUMBER := 0;
   V_ERR_NUM       					NUMBER;
   
	 V_PERCENT_DISP		    		FLOAT  := 0;
   
   V_PARAM_ENTRADA 					VARCHAR2(500);
   V_ERR_TXT       					VARCHAR2(100);
   V_CAMP_NULL      				VARCHAR2(20);
   V_NOM_PKG             		VARCHAR2(50):= 'PKG_TAULES_DW.SP_DISPONIBILITAT_DIA';
   
   V_REGISTRE								REGISTRE%ROWTYPE;
	 V_DISPONIBILITAT_DIA			DISPONIBILITAT_DIA%ROWTYPE;

   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_REGISTRE '||P_ID_REGISTRE;      

     -- Verifiquem que el registre existeixi.
     IF (P_ID_REGISTRE IS NULL)  THEN
         V_CAMP_NULL := 'ID_REGISTRE';
         RAISE EXC_CAMP_NULL;
     END IF;
      
     -- Obtenim les dades del registre.
     SELECT *
       INTO V_REGISTRE				
       FROM REGISTRE
      WHERE ID_REGISTRE = P_ID_REGISTRE;

  		V_ANY := EXTRACT(YEAR FROM V_REGISTRE.DATA_INICI);
  		V_MES := EXTRACT(MONTH FROM V_REGISTRE.DATA_INICI);
  		V_DIA := EXTRACT(DAY FROM V_REGISTRE.DATA_INICI);
  		
  		IF V_REGISTRE.IND_DISPONIBLE = 'S'	THEN
  				V_REG_DISP 		    := 1;
  				V_PERCENT_DISP    := 100;
  		ELSE
  				V_REG_NO_DISP 	  := 1;
  		END IF;    
  		
      ----------------------------------------------------------------------------------------
		  -- ACTUALITZACIO DE LA TAULA DISPONIBILITAT_DIA.
      ----------------------------------------------------------------------------------------

      -- Si no existeix registre a DISPONIBILITAT_DIA per l'any-mes-dia, el donem
      -- d'alta. Si ja existeix, la modifiquem considerant les noves dades.
       
 			SELECT COUNT(*)
  		INTO	V_COUNT
  		FROM	DISPONIBILITAT_DIA
  		WHERE	ANYO	= V_ANY
  		AND		MES		= V_MES
  		AND		DIA		= V_DIA;

  		IF (V_COUNT = 1) THEN
      
        SELECT *
        INTO	V_DISPONIBILITAT_DIA
        FROM	DISPONIBILITAT_DIA
        WHERE	ANYO	= V_ANY
        AND		MES		= V_MES
  			AND		DIA		= V_DIA;
        
        V_DISPONIBILITAT_DIA.REGISTRES_TOT 			:= V_DISPONIBILITAT_DIA.REGISTRES_TOT 			+ 1;
        V_DISPONIBILITAT_DIA.REGISTRES_DISP 		:= V_DISPONIBILITAT_DIA.REGISTRES_DISP 		  + V_REG_DISP;
        V_DISPONIBILITAT_DIA.REGISTRES_NO_DISP 	:= V_DISPONIBILITAT_DIA.REGISTRES_NO_DISP 	+ V_REG_NO_DISP;
        
        IF V_DISPONIBILITAT_DIA.REGISTRES_DISP	> 0	THEN
        	V_DISPONIBILITAT_DIA.PERCENTATGE_DISP 	:= ROUND((V_DISPONIBILITAT_DIA.REGISTRES_DISP / V_DISPONIBILITAT_DIA.REGISTRES_TOT) * 100, 3);
        END IF;
        
        V_PERCENT_DISP := V_DISPONIBILITAT_DIA.PERCENTATGE_DISP;	-- Percentatge de disponibilitat del dia.

				UPDATE	DISPONIBILITAT_DIA SET
        				ROW = V_DISPONIBILITAT_DIA
         WHERE 	ID_DISPONIBILITAT_DIA = V_DISPONIBILITAT_DIA.ID_DISPONIBILITAT_DIA;
         
  		ELSE
        
	      INSERT INTO DISPONIBILITAT_DIA (ANYO, MES, DIA, DATA_CALCUL, REGISTRES_TOT, REGISTRES_DISP, 
	      												 REGISTRES_NO_DISP,PERCENTATGE_DISP)
	      VALUES(V_ANY, V_MES, V_DIA, SYSDATE, 1, V_REG_DISP, V_REG_NO_DISP, V_PERCENT_DISP);
  		END IF;  		 
      
     ----------------------------------------------------------------------------------------
     -- ACTUALITZEM LES DADES ESTADISTIQUES DE DW_DIA_MENYS_DISP.
     ----------------------------------------------------------------------------------------
     
      -- Comprovem si el nostre dia es el que te menys disponibilitat en el mes.
      SP_DW_DIA_MENYS_DISP(V_ANY, V_MES, V_DIA, V_PERCENT_DISP, RSP);

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGISTRE_NOEXIST THEN
        RSP := 'ERROR: El Registre amb identificador: ' || P_ID_REGISTRE || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

   END SP_DISPONIBILITAT_DIA;

  /******************************************************************************
     Nom:                 SP_DW_DIA_MENYS_DISP
     Propsit:            Actualitza l'estadistica de la taula DW_DIA_MENYS_DISP. 
                     
     Parmetres entrada:  -	P_ANY 								NUMBER(10): any en el que s'ha modificat el registre.
                          -	P_MES 								NUMBER(10): mes en el que s'ha modificat el registre.
                          -	P_DIA 								NUMBER(10): dia en el que s'ha modificat el registre.
                          -	P_PERCENTATGE_DISP  	FLOAT			: percentatge de disponibilitat del dia.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_DW_DIA_MENYS_DISP (P_ANY         			IN DW_DIA_MENYS_DISP.ANYO%TYPE, 
                                  P_MES         			IN DW_DIA_MENYS_DISP.MES%TYPE, 
                                  P_DIA         			IN DW_DIA_MENYS_DISP.DIA%TYPE, 
                                  P_PERCENTATGE_DISP  IN DW_DIA_MENYS_DISP.PERCENTATGE_DISP%TYPE, 
                                  RSP           			OUT VARCHAR2)
  IS
    
    EXC_CAMP_NULL   					EXCEPTION;
    
    V_ID_DW_DIA_MENYS_DISP  	DW_DIA_MENYS_DISP.ID_DW_DIA_MENYS_DISP%TYPE;
    
    V_COUNT                   NUMBER(10) := '0';
    V_ERR_NUM       				  NUMBER;
    
    V_MIN_PERCENTATGE_DISP		FLOAT;
    
    V_INSERTAR								BOOLEAN := FALSE;
   
    V_PARAM_ENTRADA 				  VARCHAR2(500);
    V_ERR_TXT       				  VARCHAR2(100);
    V_CAMP_NULL      				  VARCHAR2(20);
    V_NOM_PKG             	  VARCHAR2(50):= 'PKG_TAULES_DW.SP_DW_DIA_MENYS_DISP';
    
    BEGIN 
    
     V_PARAM_ENTRADA := 	'P_ANY '||P_ANY||', P_MES '||P_MES||', P_DIA '||P_DIA||
                          ', P_PERCENTATGE_DISP '||P_PERCENTATGE_DISP;      

      -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_ANY IS NULL)  THEN
         V_CAMP_NULL := 'P_ANY';
         RAISE EXC_CAMP_NULL;
      END IF;
     IF (P_MES IS NULL)  THEN
         V_CAMP_NULL := 'P_MES';
         RAISE EXC_CAMP_NULL;
      END IF;
     IF (P_DIA IS NULL)  THEN
         V_CAMP_NULL := 'P_DIA';
         RAISE EXC_CAMP_NULL;
      END IF;
     IF (P_PERCENTATGE_DISP IS NULL)  THEN
         V_CAMP_NULL := 'P_PERCENTATGE_DISP';
         RAISE EXC_CAMP_NULL;
      END IF;
      
      -- Verifiquem si hi han dades per l'any-mes.
    SELECT COUNT(*)
     INTO  V_COUNT
     FROM  DW_DIA_MENYS_DISP
    WHERE  ANYO  = P_ANY 
      AND  MES   = P_MES;
    
    IF V_COUNT > 0 THEN
        -- Obtenim i comparem el dia amb menys disponibilitat en el mes amb el dia 
        -- amb el que venim. Pot haver mes d'un dia amb el mateix % de disponibilitat. 
        SELECT MIN(PERCENTATGE_DISP)
         INTO  V_MIN_PERCENTATGE_DISP
         FROM  DW_DIA_MENYS_DISP
        WHERE  ANYO  = P_ANY 
          AND  MES   = P_MES;
    
        IF V_MIN_PERCENTATGE_DISP > P_PERCENTATGE_DISP THEN
        		-- Si el % del dia amb que venim es inferior als existents, sustituim 
        		-- l'estadistica de l'any-mes per la del dia amb que venim.
				    DELETE FROM  DW_DIA_MENYS_DISP
				    WHERE  ANYO  = P_ANY 
				      AND  MES   = P_MES;
				      
    				V_INSERTAR := TRUE;
				      
        ELSE 
            IF V_MIN_PERCENTATGE_DISP = P_PERCENTATGE_DISP THEN
              -- Si el % del dia amb que venim es igual als existents, insertem 
              -- les dades del dia amb que venim, sempre que no existeixi ja.
                SELECT COUNT(*)
                 INTO  V_COUNT
                 FROM  DW_DIA_MENYS_DISP
                WHERE  ANYO = P_ANY 
                  AND  MES  = P_MES
                  AND  DIA  = P_DIA;
                  
                IF V_COUNT = 0  THEN
                  V_INSERTAR := TRUE;
                END IF;            
            END IF;
        END IF;
    ELSE
    		V_INSERTAR := TRUE;
    END IF;
    
    IF (V_INSERTAR = TRUE) THEN 
      -- Donem d'alta un registre a la taula.
	      INSERT INTO DW_DIA_MENYS_DISP (ANYO, MES, DIA, DATA_CALCUL, PERCENTATGE_DISP)
	           VALUES(P_ANY, P_MES, P_DIA, SYSDATE, P_PERCENTATGE_DISP);
    END IF;

    
   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
    
  END SP_DW_DIA_MENYS_DISP; 
  
  /******************************************************************************
     Nom:                 SP_DW_CLIENT_MENYS_DISP
     Propsit:            Actualitza la taula DW_CLIENT_MENYS_DISP. 
                     
     Parmetres entrada:  -	P_ID_REGISTRE NUMBER(10): identificaci del registre.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_DW_CLIENT_MENYS_DISP (P_ID_REGISTRE   IN REGISTRE.ID_REGISTRE%TYPE,
                              			  RSP             OUT VARCHAR2)                        		 
    IS

   EXC_CAMP_NULL         		EXCEPTION;
   EXC_REGISTRE_NOEXIST			EXCEPTION;

	 V_ANY						    		NUMBER;
	 V_MES						    		NUMBER;
	 V_CLIENT			    				NUMBER;
	 V_SERVEI_WEB			    		NUMBER;
	 V_COUNT					    		NUMBER := 0;
	 V_REG_DISP				    		NUMBER := 0;
	 V_REG_NO_DISP		    		NUMBER := 0;
   V_ERR_NUM       					NUMBER;
   V_REGISTRES_TOT       		NUMBER;
   V_REGISTRES_DISP       	NUMBER;
   
	 V_PERCENTATGE_DISP    		FLOAT  := 0;
	 V_MIN_PERCENTATGE_DISP		FLOAT  := 0;
	 
   V_INSERTAR								BOOLEAN := FALSE;
   
   V_PARAM_ENTRADA 					VARCHAR2(500);
   V_ERR_TXT       					VARCHAR2(100);
   V_CAMP_NULL      				VARCHAR2(20);
   V_NOM_PKG             		VARCHAR2(50):= 'PKG_TAULES_DW.SP_DW_CLIENT_MENYS_DISP';
   
   V_REGISTRE								REGISTRE%ROWTYPE;

   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_REGISTRE '||P_ID_REGISTRE;      

     -- Verifiquem que el registre existeixi.
     IF (P_ID_REGISTRE IS NULL)  THEN
         V_CAMP_NULL := 'ID_REGISTRE';
         RAISE EXC_CAMP_NULL;
     END IF;
      
     -- Obtenim les dades del registre.
     SELECT *
       INTO V_REGISTRE				
       FROM REGISTRE
      WHERE ID_REGISTRE = P_ID_REGISTRE;

  		V_ANY 				:= EXTRACT(YEAR FROM V_REGISTRE.DATA_INICI);
  		V_MES 				:= EXTRACT(MONTH FROM V_REGISTRE.DATA_INICI);
  		V_SERVEI_WEB 	:= V_REGISTRE.SERVEI_WEB;
  		
  		SELECT CLIENT
  		  INTO V_CLIENT
  		  FROM SERVEI_WEB
  		 WHERE ID_SERVEI_WEB = V_SERVEI_WEB;
  		 
     IF (V_CLIENT IS NULL)  THEN
         V_CAMP_NULL := 'V_CLIENT';
         RAISE EXC_CAMP_NULL;
     END IF;

     -- Calculem el % de disponibilitat del client en el mes.
  		 
      SELECT SUM(REGISTRES_TOT),
       			 SUM(REGISTRES_DISP)
        INTO	V_REGISTRES_TOT,
        			V_REGISTRES_DISP
        FROM	DISPONIBILITAT_MES,
        			SERVEI_WEB
        WHERE	SERVEI_WEB.ID_SERVEI_WEB 	= DISPONIBILITAT_MES.SERVEI_WEB
        AND		SERVEI_WEB.CLIENT					= V_CLIENT
        AND		ANYO											= V_ANY
        AND		MES												= V_MES;

      IF V_REGISTRES_DISP	> 0	THEN
      	 V_PERCENTATGE_DISP 	:= ROUND((V_REGISTRES_DISP / V_REGISTRES_TOT) * 100, 3);
      END IF;

      ----------------------------------------------------------------------------------------
		  -- ACTUALITZACIO DE LA TAULA DW_CLIENT_MENYS_DISP.
      ----------------------------------------------------------------------------------------

      -- Verifiquem si hi han dades per l'any-mes.
	    SELECT COUNT(*)
	     INTO  V_COUNT
	     FROM  DW_CLIENT_MENYS_DISP
	    WHERE  ANYO  = V_ANY 
	      AND  MES   = V_MES;
	    
	    IF V_COUNT > 0 THEN
	        -- Obtenim i comparem el client amb menys disponibilitat en el mes amb el client 
	        -- amb el que venim. Pot haver mes d'un client amb el mateix % de disponibilitat. 
	        SELECT MIN(PERCENTATGE_DISP)
	         INTO  V_MIN_PERCENTATGE_DISP
	         FROM  DW_CLIENT_MENYS_DISP
	        WHERE  ANYO  = V_ANY 
	          AND  MES   = V_MES;
	    
	        IF V_MIN_PERCENTATGE_DISP > V_PERCENTATGE_DISP THEN
	        		-- Si el % del client amb que venim es inferior als existents, sustituim 
	        		-- l'estadistica de l'any-mes per la del client amb que venim.
					    DELETE FROM  DW_CLIENT_MENYS_DISP
					    WHERE  ANYO  = V_ANY 
					      AND  MES   = V_MES;
					      
	    				V_INSERTAR := TRUE;
					      
	        ELSE 
	            IF V_MIN_PERCENTATGE_DISP = V_PERCENTATGE_DISP THEN
	              -- Si el % del client amb que venim es igual als existents, insertem 
	              -- les dades del client amb que venim, sempre que no existeixi ja.
                SELECT COUNT(*)
                 INTO  V_COUNT
                 FROM  DW_CLIENT_MENYS_DISP
                WHERE  ANYO   = V_ANY 
                  AND  MES    = V_MES
                  AND  CLIENT = V_CLIENT;
                  
                IF V_COUNT = 0  THEN
                  V_INSERTAR := TRUE;
                END IF;
	            END IF;
	        END IF;
	    ELSE
	    		V_INSERTAR := TRUE;
	    END IF;
	    
	    IF (V_INSERTAR = TRUE) THEN 
	      -- Donem d'alta un registre a la taula.
		      INSERT INTO DW_CLIENT_MENYS_DISP (ANYO, MES, CLIENT, DATA_CALCUL, PERCENTATGE_DISP)
		           VALUES(V_ANY, V_MES, V_CLIENT, SYSDATE, V_PERCENTATGE_DISP);
	    END IF;

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGISTRE_NOEXIST THEN
        RSP := 'ERROR: El Registre amb identificador: ' || P_ID_REGISTRE || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

   END SP_DW_CLIENT_MENYS_DISP;

  
  /******************************************************************************
     Nom:                 SP_DW_REGLA_MES_TEMPS
     Propsit:            Actualitza la taula DW_REGLA_MES_TEMPS. 
                     
     Parmetres entrada:  -	P_ID_REGISTRE NUMBER(10): identificaci del registre.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_DW_REGLA_MES_TEMPS (P_ID_REGISTRE   IN REGISTRE.ID_REGISTRE%TYPE,
                              			RSP             OUT VARCHAR2)                        		 
    IS

   EXC_CAMP_NULL         		EXCEPTION;
   EXC_REGISTRE_NOEXIST			EXCEPTION;

	 V_ANY						    		NUMBER;
	 V_MES						    		NUMBER;
	 V_REGLA			    				NUMBER;
	 V_SERVEI_WEB			    		NUMBER;
	 V_COUNT					    		NUMBER := 0;
	 V_REG_DISP				    		NUMBER := 0;
   V_ERR_NUM       					NUMBER;
   V_REGISTRES_DISP       	NUMBER;
   V_TEMPS_EXECUCIO_DISP 		NUMBER;
   
   V_MAX_TEMPS_MIG 					FLOAT  := 0;
   V_TEMPS_EXECUCIO_MIG 		FLOAT  := 0;
   
   V_INSERTAR								BOOLEAN := FALSE;
   
   V_PARAM_ENTRADA 					VARCHAR2(500);
   V_ERR_TXT       					VARCHAR2(100);
   V_CAMP_NULL      				VARCHAR2(20);
   V_NOM_PKG             		VARCHAR2(50):= 'PKG_TAULES_DW.SP_DW_REGLA_MES_TEMPS';
   
   V_REGISTRE								REGISTRE%ROWTYPE;

   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_REGISTRE '||P_ID_REGISTRE;      

     -- Verifiquem que el registre existeixi.
     IF (P_ID_REGISTRE IS NULL)  THEN
         V_CAMP_NULL := 'ID_REGISTRE';
         RAISE EXC_CAMP_NULL;
     END IF;
      
     -- Obtenim les dades del registre.
     SELECT *
       INTO V_REGISTRE				
       FROM REGISTRE
      WHERE ID_REGISTRE = P_ID_REGISTRE;

  		V_ANY 				:= EXTRACT(YEAR FROM V_REGISTRE.DATA_INICI);
  		V_MES 				:= EXTRACT(MONTH FROM V_REGISTRE.DATA_INICI);
  		V_SERVEI_WEB 	:= V_REGISTRE.SERVEI_WEB;
  		V_REGLA			 	:= V_REGISTRE.REGLA;
  		
     -- Calculem el temps mig d'execucio de la regla en el mes.
  		 
      SELECT SUM(REGISTRES_DISP),
       			 SUM(TEMPS_EXECUCIO_DISP)
        INTO	V_REGISTRES_DISP,
        			V_TEMPS_EXECUCIO_DISP
        FROM	DISPONIBILITAT_MES
        WHERE	ANYO				= V_ANY
        AND		MES					= V_MES
        AND		SERVEI_WEB	= V_SERVEI_WEB
        AND		REGLA				= V_REGLA;

      IF V_REGISTRES_DISP	> 0	THEN
      	 V_TEMPS_EXECUCIO_DISP 	:= ROUND((V_TEMPS_EXECUCIO_DISP / V_REGISTRES_DISP), 3);
      END IF;

      ----------------------------------------------------------------------------------------
		  -- ACTUALITZACIO DE LA TAULA DW_REGLA_MES_TEMPS.
      ----------------------------------------------------------------------------------------

      -- Verifiquem si hi han dades per l'any-mes.
	    SELECT COUNT(*)
	     INTO  V_COUNT
	     FROM  DW_REGLA_MES_TEMPS
	    WHERE  ANYO  = V_ANY 
	      AND  MES   = V_MES;
	    
	    IF V_COUNT > 0 THEN
	        -- Obtenim i comparem la regla que mes temps mig triga en el mes amb la regla 
	        -- amb la que venim. Pot haver mes d'una regla amb el mateix temps d'execucio. 
	        SELECT MAX(TEMPS_MIG)
	         INTO  V_MAX_TEMPS_MIG
	         FROM  DW_REGLA_MES_TEMPS
	        WHERE  ANYO  = V_ANY 
	          AND  MES   = V_MES;
	    
	        IF V_TEMPS_EXECUCIO_DISP > V_MAX_TEMPS_MIG THEN
	        		-- Si el temps d'execucio de la regla amb que venim es superior als existents,  
	        		-- sustituim l'estadistica de l'any-mes per la de la regla amb que venim.
					    DELETE FROM  DW_REGLA_MES_TEMPS
					    WHERE  ANYO  = V_ANY 
					      AND  MES   = V_MES;
					      
	    				V_INSERTAR := TRUE;
					      
	        ELSE 
	            IF V_TEMPS_EXECUCIO_DISP = V_MAX_TEMPS_MIG THEN
	              -- Si el temps d'execucio de la regla amb que venim es igual als existents,  
	              -- insertem les dades de la regla amb que venim, sempre que no existeixi ja.
                SELECT COUNT(*)
                 INTO  V_COUNT
                 FROM  DW_REGLA_MES_TEMPS
                WHERE  ANYO  = V_ANY 
                  AND  MES   = V_MES
                  AND  REGLA = V_REGLA;
                  
                IF V_COUNT = 0  THEN
                  V_INSERTAR := TRUE;
                END IF;
	            END IF;
	        END IF;
	    ELSE
	    		V_INSERTAR := TRUE;
	    END IF;
	    
	    IF (V_INSERTAR = TRUE) THEN 
	      -- Donem d'alta un registre a la taula.
		      INSERT INTO DW_REGLA_MES_TEMPS (ANYO, MES, DATA_CALCUL, REGLA, TEMPS_MIG)
		           VALUES(V_ANY, V_MES, SYSDATE, V_REGLA, V_TEMPS_EXECUCIO_DISP);
	    END IF;

      -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGISTRE_NOEXIST THEN
        RSP := 'ERROR: El Registre amb identificador: ' || P_ID_REGISTRE || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

   END SP_DW_REGLA_MES_TEMPS;
      
END PKG_TAULES_DW;
/

spool off;