/*
* Creaci del package PKG_PAS.
* S'ha d'executar connectat com a monitor/monitor1606
*/

spool 13_PackagePas.log;

CREATE OR REPLACE PACKAGE PKG_PAS AS

  -- Procediment per donar d'alta un Pas. 
  PROCEDURE SP_ALTA_PAS 		 	 (P_REGLA      				IN PAS.REGLA%TYPE,
                            	 	P_ORDRE      				IN PAS.ORDRE%TYPE,
                            	 	P_DATA_ALTA      		IN PAS.DATA_ALTA%TYPE,
                            	 	P_DESCRIPCIO      	IN PAS.DESCRIPCIO%TYPE,
                            		P_URL_SERVEI    		IN PAS.URL_SERVEI%TYPE,
                            		P_TEMPS_MAXIM    		IN PAS.TEMPS_MAXIM%TYPE,
                            		P_PETICIO_SERVEI    IN PAS.PETICIO_SERVEI%TYPE,
                            		P_RESPOSTA_ESPERADA IN PAS.RESPOSTA_ESPERADA%TYPE,
                            		P_ESTAT					  	IN PAS.ESTAT%TYPE,
                            		P_DATA_ESTAT		  	IN PAS.DATA_ESTAT%TYPE,
                            		RSP               	OUT VARCHAR2);
                                 
	-- Procediment per donar de baixa lgica un Pas, a partir de l'identificador passat per parametre.
  PROCEDURE SP_BAIXA_PAS 		   (P_ID_PAS 						IN PAS.ID_PAS%TYPE,
                            		P_DATA_ESTAT		  	IN PAS.DATA_ESTAT%TYPE,
  															RSP              		OUT VARCHAR2);

	-- Procediment per actualitzar un Pas.
  PROCEDURE SP_MODIFICA_PAS	   (P_ID_PAS	      		IN PAS.ID_PAS%TYPE,
                            	 	P_REGLA      				IN PAS.REGLA%TYPE,
                            	 	P_ORDRE      				IN PAS.ORDRE%TYPE,
                            	 	P_DATA_ALTA      		IN PAS.DATA_ALTA%TYPE,
                            	 	P_DESCRIPCIO      	IN PAS.DESCRIPCIO%TYPE,
                            		P_URL_SERVEI    		IN PAS.URL_SERVEI%TYPE,
                            		P_TEMPS_MAXIM    		IN PAS.TEMPS_MAXIM%TYPE,
                            		P_PETICIO_SERVEI    IN PAS.PETICIO_SERVEI%TYPE,
                            		P_RESPOSTA_ESPERADA IN PAS.RESPOSTA_ESPERADA%TYPE,
                            		P_ESTAT					  	IN PAS.ESTAT%TYPE,
                            		P_DATA_ESTAT		  	IN PAS.DATA_ESTAT%TYPE,
                            		RSP               	OUT VARCHAR2);

END PKG_PAS;
/


CREATE OR REPLACE PACKAGE BODY PKG_PAS AS

  /******************************************************************************
     Nom:                 SP_ALTA_PAS
     Propsit:            Dona d'alta un Pas de monitoritzaci, a la taula PAS. 
                     
     Parmetres entrada:  -	P_REGLA NUMBER(10): regla a la que pertany el Pas.
     											-	P_ORDRE NUMBER(5): ordre d'execuci del Pas dins la regla.
     											-	P_DATA_ALTA DATE: data d'alta del Pas.
                          -	P_DESCRIPCIO VARCHAR2(150): descripci del Pas.
                          -	P_URL_SERVEI  VARCHAR2(200): URL del servei web al que crida el Pas.
                          -	P_TEMPS_MAXIM NUMBER(10): temps mxim d'espera de resposta del servei web (milisegons). 
                          -	P_PETICIO_SERVEI VARCHAR2(250)): petici que s'enviar al servei web. 
                          -	P_RESPOSTA_ESPERADA VARCHAR2(250): resposta esperada del servei web. 
                          -	P_ESTAT CHAR(1): estat del Pas (A, D). 
                          -	P_DATA_ESTAT DATE: data d'inici de l'estat del Pas. 
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_ALTA_PAS 		 	 (P_REGLA      				IN PAS.REGLA%TYPE,
                            	 	P_ORDRE      				IN PAS.ORDRE%TYPE,
                            	 	P_DATA_ALTA      		IN PAS.DATA_ALTA%TYPE,
                            	 	P_DESCRIPCIO      	IN PAS.DESCRIPCIO%TYPE,
                            		P_URL_SERVEI    		IN PAS.URL_SERVEI%TYPE,
                            		P_TEMPS_MAXIM    		IN PAS.TEMPS_MAXIM%TYPE,
                            		P_PETICIO_SERVEI    IN PAS.PETICIO_SERVEI%TYPE,
                            		P_RESPOSTA_ESPERADA IN PAS.RESPOSTA_ESPERADA%TYPE,
                            		P_ESTAT					  	IN PAS.ESTAT%TYPE,
                            		P_DATA_ESTAT		  	IN PAS.DATA_ESTAT%TYPE,
                            		RSP               	OUT VARCHAR2)
    IS

   EXC_CAMP_NULL         	EXCEPTION;
   EXC_PAS_EXIST					EXCEPTION;
   EXC_REGLA_NOEXIST			EXCEPTION;

   V_COUNT								NUMBER;
   V_ERR_NUM       				NUMBER;
   
   V_PARAM_ENTRADA 				VARCHAR2(500);
   V_ERR_TXT       				VARCHAR2(100);
   V_CAMP_NULL      			VARCHAR2(20);
   
   V_NOM_PKG             	VARCHAR2(50):= 'PKG_PAS.SP_ALTA_PAS';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_REGLA '||P_REGLA||', P_ORDRE: '||P_ORDRE||', P_DATA_ALTA: '||P_DATA_ALTA||
                      		', P_DESCRIPCIO: '||P_DESCRIPCIO||', P_URL_SERVEI: '||P_URL_SERVEI||
                      		', P_TEMPS_MAXIM: '||P_TEMPS_MAXIM||', P_PETICIO_SERVEI: '||P_PETICIO_SERVEI||
                      		', P_RESPOSTA_ESPERADA: '||P_RESPOSTA_ESPERADA||', P_ESTAT: '||P_ESTAT||
                      		', P_DATA_ESTAT: '||P_DATA_ESTAT;      

     -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_REGLA IS NULL)  THEN
         V_CAMP_NULL := 'P_REGLA';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_ORDRE IS NULL)  THEN
         V_CAMP_NULL := 'P_ORDRE';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_DATA_ALTA IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ALTA';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (TRIM(P_DESCRIPCIO) IS NULL)  THEN
         V_CAMP_NULL := 'P_DESCRIPCIO';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_URL_SERVEI) IS NULL)  THEN
         V_CAMP_NULL := 'P_URL_SERVEI';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_TEMPS_MAXIM IS NULL)  THEN
         V_CAMP_NULL := 'P_TEMPS_MAXIM';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_PETICIO_SERVEI) IS NULL)  THEN
         V_CAMP_NULL := 'P_PETICIO_SERVEI';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_RESPOSTA_ESPERADA) IS NULL)  THEN
         V_CAMP_NULL := 'P_RESPOSTA_ESPERADA';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_ESTAT) IS NULL)  THEN
         V_CAMP_NULL := 'P_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_DATA_ESTAT IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;

     -- Verifiquem que no existeixi ja un Pas amb la mateixa clau.
     BEGIN

       SELECT COUNT(*)
         INTO V_COUNT
         FROM PAS
        WHERE REGLA  = P_REGLA
          AND ORDRE  = P_ORDRE;          
                  
        IF V_COUNT > 0	THEN
        	RAISE EXC_PAS_EXIST;
				END IF;
     END;
     
     -- Verifiquem que la regla existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGLA
        WHERE ID_REGLA = P_REGLA;

        IF V_COUNT = 0	THEN
           RAISE EXC_REGLA_NOEXIST;
        END IF;
      END;
      
      -- Insertem el PAS.
      INSERT INTO PAS 
           (REGLA, ORDRE, DATA_ALTA, DESCRIPCIO, URL_SERVEI, TEMPS_MAXIM, PETICIO_SERVEI, 
            RESPOSTA_ESPERADA, ESTAT, DATA_ESTAT)
      VALUES(P_REGLA, P_ORDRE, P_DATA_ALTA, P_DESCRIPCIO, P_URL_SERVEI, P_TEMPS_MAXIM, P_PETICIO_SERVEI, 
            P_RESPOSTA_ESPERADA, P_ESTAT, P_DATA_ESTAT);

       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_PAS_EXIST THEN
        RSP := 'ERROR: Ja existeix un Pas per a la Regla: ' || P_REGLA || ' i Ordre: ' || P_ORDRE;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGLA_NOEXIST THEN
        RSP := 'ERROR: La Regla amb identificador: ' || P_REGLA || ' no existeix';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_ALTA_PAS;
   
  /******************************************************************************
     Nom:                 SP_BAIXA_PAS
     Propsit:            Dona de baixa logica un Pas. 
                     
     Parmetres entrada:  -	P_ID_PAS NUMBER(10): identificaci del Pas.
                          -	P_DATA_ESTAT DATE: data d'inici de l'estat del Pas. 
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_BAIXA_PAS 	(P_ID_PAS  	    IN PAS.ID_PAS%TYPE,
                             P_DATA_ESTAT		IN PAS.DATA_ESTAT%TYPE,
                             RSP            OUT VARCHAR2) 
    IS

   EXC_CAMP_NULL         	EXCEPTION;
   EXC_PAS_NOEXIST				EXCEPTION;
   EXC_PAS_BAIXA					EXCEPTION;

   V_COUNT								NUMBER;
   V_ERR_NUM       				NUMBER;
   
   V_PARAM_ENTRADA 				VARCHAR2(500);
   V_ERR_TXT       				VARCHAR2(100);
   V_CAMP_NULL      			VARCHAR2(20);
   
   V_NOM_PKG             	VARCHAR2(50):= 'PKG_PAS.SP_BAIXA_PAS';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_PAS: '||P_ID_PAS||', P_DATA_ESTAT: '||P_DATA_ESTAT;      

     -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_ID_PAS IS NULL)  THEN
         V_CAMP_NULL := 'ID_PAS';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_DATA_ESTAT IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;

     -- Verifiquem que el Pas existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM PAS
        WHERE ID_PAS = P_ID_PAS;

        IF V_COUNT = 0	THEN
           RAISE EXC_PAS_NOEXIST;
        END IF;
      END;
      
     -- Verifiquem que el Pas no estigui actualment donat de baixa.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM PAS
        WHERE ID_PAS = P_ID_PAS
          AND	ESTAT		 = 'B';

        IF V_COUNT > 0	THEN
           RAISE EXC_PAS_BAIXA;
        END IF;
      END;
      
      -- Donem de baixa lgica el Pas.
      UPDATE PAS SET 
      			 ESTAT 			= 'B',
      			 DATA_ESTAT = P_DATA_ESTAT
      WHERE  ID_PAS 	= P_ID_PAS;

      RSP := 'OK';

       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_PAS_NOEXIST THEN
        RSP := 'ERROR: El Pas amb identificador: ' || P_ID_PAS || ' no existeix';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_PAS_BAIXA THEN
        RSP := 'ERROR: El Pas amb identificador: ' || P_ID_PAS || ' ja est donat de baixa';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_BAIXA_PAS;
   
  /******************************************************************************
     Nom:                 SP_MODIFICA_PAS
     Propsit:            Actualitza un Pas (taula PAS). 
                     
     Parmetres entrada:  -	P_ID_PAS NUMBER(10): identificaci del Pas.
     											-	P_REGLA NUMBER(10): regla a la que pertany el Pas.
     											-	P_ORDRE NUMBER(5): ordre d'execuci del Pas dins la regla.
     											-	P_DATA_ALTA DATE: data d'alta del Pas.
                          -	P_DESCRIPCIO VARCHAR2(150): descripci del Pas.
                          -	P_URL_SERVEI  VARCHAR2(200): URL del servei web al que crida el Pas.
                          -	P_TEMPS_MAXIM NUMBER(10): temps mxim d'espera de resposta del servei web (milisegons). 
                          -	P_PETICIO_SERVEI VARCHAR2(250)): petici que s'enviar al servei web. 
                          -	P_RESPOSTA_ESPERADA VARCHAR2(250): resposta esperada del servei web. 
                          -	P_ESTAT CHAR(1): estat del Pas (A, D). 
                          -	P_DATA_ESTAT DATE: data d'inici de l'estat del Pas. 
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_MODIFICA_PAS	 	 (P_ID_PAS	      		IN PAS.ID_PAS%TYPE,
 												 		 	  P_REGLA      				IN PAS.REGLA%TYPE,
                            	 	P_ORDRE      				IN PAS.ORDRE%TYPE,
                            	 	P_DATA_ALTA      		IN PAS.DATA_ALTA%TYPE,
                            	 	P_DESCRIPCIO      	IN PAS.DESCRIPCIO%TYPE,
                            		P_URL_SERVEI    		IN PAS.URL_SERVEI%TYPE,
                            		P_TEMPS_MAXIM    		IN PAS.TEMPS_MAXIM%TYPE,
                            		P_PETICIO_SERVEI    IN PAS.PETICIO_SERVEI%TYPE,
                            		P_RESPOSTA_ESPERADA IN PAS.RESPOSTA_ESPERADA%TYPE,
                            		P_ESTAT					  	IN PAS.ESTAT%TYPE,
                            		P_DATA_ESTAT		  	IN PAS.DATA_ESTAT%TYPE,
                            		RSP               	OUT VARCHAR2)
    IS

   EXC_CAMP_NULL         	EXCEPTION;
   EXC_PAS_EXIST					EXCEPTION;
   EXC_REGLA_NOEXIST			EXCEPTION;
   EXC_PAS_NOEXIST				EXCEPTION;

   V_COUNT								NUMBER;
   V_ERR_NUM       				NUMBER;
   
   V_PARAM_ENTRADA 				VARCHAR2(500);
   V_ERR_TXT       				VARCHAR2(100);
   V_CAMP_NULL      			VARCHAR2(20);
   
   V_NOM_PKG             	VARCHAR2(50):= 'PKG_PAS.SP_MODIFICA_PAS';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_PAS '||P_ID_PAS||', P_REGLA '||P_REGLA||', P_ORDRE: '||P_ORDRE||
     											', P_DATA_ALTA: '||P_DATA_ALTA||', P_DESCRIPCIO: '||P_DESCRIPCIO||
                      		', P_URL_SERVEI: '||P_URL_SERVEI||', P_TEMPS_MAXIM: '||P_TEMPS_MAXIM||
                      		', P_PETICIO_SERVEI: '||P_PETICIO_SERVEI||', P_RESPOSTA_ESPERADA: '||P_RESPOSTA_ESPERADA||
                      		', P_ESTAT: '||P_ESTAT||', P_DATA_ESTAT: '||P_DATA_ESTAT;      

      -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_ID_PAS IS NULL)  THEN
         V_CAMP_NULL := 'ID_PAS';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_REGLA IS NULL)  THEN
         V_CAMP_NULL := 'P_REGLA';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_ORDRE IS NULL)  THEN
         V_CAMP_NULL := 'P_ORDRE';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_DATA_ALTA IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ALTA';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (TRIM(P_DESCRIPCIO) IS NULL)  THEN
         V_CAMP_NULL := 'P_DESCRIPCIO';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_URL_SERVEI) IS NULL)  THEN
         V_CAMP_NULL := 'P_URL_SERVEI';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_TEMPS_MAXIM IS NULL)  THEN
         V_CAMP_NULL := 'P_TEMPS_MAXIM';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_PETICIO_SERVEI) IS NULL)  THEN
         V_CAMP_NULL := 'P_PETICIO_SERVEI';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_RESPOSTA_ESPERADA) IS NULL)  THEN
         V_CAMP_NULL := 'P_RESPOSTA_ESPERADA';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_ESTAT) IS NULL)  THEN
         V_CAMP_NULL := 'P_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_DATA_ESTAT IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     -- Verifiquem que el Pas existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM PAS
        WHERE ID_PAS = P_ID_PAS;

        IF V_COUNT = 0	THEN
           RAISE EXC_PAS_NOEXIST;
        END IF;
      END;
      
      -- Verifiquem que no existeixi ja un Pas, diferent a la que volem actualitzar,
      -- amb la mateixa clau.
     BEGIN
       SELECT COUNT(*)
         INTO V_COUNT
         FROM PAS
        WHERE REGLA  	= 	P_REGLA
          AND ORDRE   = 	P_ORDRE         
          AND ID_PAS  <> 	P_ID_PAS;          
                  
        IF V_COUNT > 0	THEN
        	RAISE EXC_PAS_EXIST;
				END IF;
     END;
     
     -- Verifiquem que la Regla existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGLA
        WHERE ID_REGLA = P_REGLA;

        IF V_COUNT = 0	THEN
           RAISE EXC_REGLA_NOEXIST;
        END IF;
      END;      
   
      -- Actualitzem el Pas.
     UPDATE PAS 
        SET  REGLA     					= P_REGLA,
        		 ORDRE     					= P_ORDRE,
        		 DATA_ALTA      		= P_DATA_ALTA,
             DESCRIPCIO   			= P_DESCRIPCIO,
             URL_SERVEI					= P_URL_SERVEI,
             TEMPS_MAXIM				= P_TEMPS_MAXIM,
             PETICIO_SERVEI			= P_PETICIO_SERVEI,
             RESPOSTA_ESPERADA	= P_RESPOSTA_ESPERADA,
             ESTAT    					= P_ESTAT,                                                                  
             DATA_ESTAT     		= P_DATA_ESTAT
     WHERE   ID_PAS	= P_ID_PAS;      
      
      RSP := 'OK';

       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_PAS_NOEXIST THEN
        RSP := 'ERROR: La PAS amb identificador: ' || P_ID_PAS || ' no existeix';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGLA_NOEXIST THEN
        RSP := 'ERROR: La Regla amb identificador: ' || P_REGLA || ' no existeix';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_PAS_EXIST THEN
        RSP := 'ERROR: Ja existeix un Pas associat a la Regla: ' || P_REGLA || ' i Ordre: ' || P_ORDRE;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_MODIFICA_PAS;
   
END PKG_PAS;
/
spool off;
