/*
* Creaci del package PKG_REGLA.
* S'ha d'executar connectat com a monitor/monitor1606
*/

spool 14_PackageRegla.log;

CREATE OR REPLACE PACKAGE PKG_REGLA AS

  -- Procediment per donar d'alta una Regla. 
  PROCEDURE SP_ALTA_REGLA 		 (P_DATA_ALTA      	IN REGLA.DATA_ALTA%TYPE,
                            	 	P_SERVEI_WEB      IN REGLA.SERVEI_WEB%TYPE,
                            	 	P_NOM      				IN REGLA.NOM%TYPE,
                            		P_PERIODICITAT    IN REGLA.PERIODICITAT%TYPE,
                            		P_ESTAT					  IN REGLA.ESTAT%TYPE,
                            		P_DATA_ESTAT		  IN REGLA.DATA_ESTAT%TYPE,
                            		RSP               OUT VARCHAR2);
                                 
	-- Procediment per donar de baixa lgica una Regla, a partir de l'identificador passat per parametre.
  PROCEDURE SP_BAIXA_REGLA 		 (P_ID_REGLA 				IN REGLA.ID_REGLA%TYPE,
                            		P_DATA_ESTAT		  IN REGLA.DATA_ESTAT%TYPE,
  															RSP              	OUT VARCHAR2);

	-- Procediment per actualitzar una Regla.
  PROCEDURE SP_MODIFICA_REGLA	 (P_ID_REGLA	      IN REGLA.ID_REGLA%TYPE,
                            	 	P_DATA_ALTA	      IN REGLA.DATA_ALTA%TYPE,
                            	 	P_SERVEI_WEB      IN REGLA.SERVEI_WEB%TYPE,
                            	 	P_NOM      				IN REGLA.NOM%TYPE,
                            		P_PERIODICITAT    IN REGLA.PERIODICITAT%TYPE,
                            		P_ESTAT					  IN REGLA.ESTAT%TYPE,
                            		P_DATA_ESTAT		  IN REGLA.DATA_ESTAT%TYPE,
                            		RSP               OUT VARCHAR2);

END PKG_REGLA;
/


CREATE OR REPLACE PACKAGE BODY PKG_REGLA AS

  /******************************************************************************
     Nom:                 SP_ALTA_REGLA
     Propsit:            Dona d'alta una regla de monitoritzaci, a la taula REGLA. 
                     
     Parmetres entrada:  -	P_DATA_ALTA DATE: data d'alta de la regla.
                          -	P_SERVEI_WEB NUMBER(10): servei web al que pertany la regla.
                          -	P_NOM  VARCHAR2(150): nom de la regla.
                          -	P_PERIODICITAT NUMBER(10): periodicitat d'execuci de la regla, en segons. 
                          -	P_ESTAT CHAR(1): estat de la regla (A, D). 
                          -	P_DATA_ESTAT DATE: data d'inici de l'estat de la regla. 
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_ALTA_REGLA 		 (P_DATA_ALTA      	IN REGLA.DATA_ALTA%TYPE,
                            	 	P_SERVEI_WEB      IN REGLA.SERVEI_WEB%TYPE,
                            	 	P_NOM      				IN REGLA.NOM%TYPE,
                            		P_PERIODICITAT    IN REGLA.PERIODICITAT%TYPE,
                            		P_ESTAT					  IN REGLA.ESTAT%TYPE,
                            		P_DATA_ESTAT		  IN REGLA.DATA_ESTAT%TYPE,
                            		RSP               OUT VARCHAR2) 
    IS

   EXC_CAMP_NULL              	EXCEPTION;
   EXC_REGLA_EXIST				      EXCEPTION;
   EXC_SERVEI_WEB_NOEXIST	      EXCEPTION;
   EXC_SERVEI_WEB_REGLA_ACTIVA  EXCEPTION;

   V_COUNT								      NUMBER;
   V_ERR_NUM       				      NUMBER;
   
   V_PARAM_ENTRADA 				      VARCHAR2(500);
   V_ERR_TXT       				      VARCHAR2(100);
   V_CAMP_NULL      			      VARCHAR2(20);
   
   V_NOM_PKG             	      VARCHAR2(50):= 'PKG_REGLA.SP_ALTA_REGLA';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_DATA_ALTA '||P_DATA_ALTA||', P_SERVEI_WEB: '||P_SERVEI_WEB||
                      		', P_NOM: '||P_NOM||', P_PERIODICITAT: '||P_PERIODICITAT||
                      		', P_ESTAT: '||P_ESTAT||', P_DATA_ESTAT: '||P_DATA_ESTAT;      

     -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_DATA_ALTA IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ALTA';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_SERVEI_WEB IS NULL)  THEN
         V_CAMP_NULL := 'SERVEI_WEB';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_NOM) IS NULL)  THEN
         V_CAMP_NULL := 'NOM';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_PERIODICITAT IS NULL)  THEN
         V_CAMP_NULL := 'PERIODICITAT';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_ESTAT) IS NULL)  THEN
         V_CAMP_NULL := 'ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_DATA_ESTAT IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;

     -- Verifiquem que no existeixi ja una regla amb la mateixa clau.
     BEGIN

       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGLA
        WHERE SERVEI_WEB  = P_SERVEI_WEB
          AND NOM  				= P_NOM;          
                  
        IF V_COUNT > 0	THEN
        	RAISE EXC_REGLA_EXIST;
				END IF;
     END;
     
     -- Verifiquem que el servei web existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM SERVEI_WEB
        WHERE ID_SERVEI_WEB = P_SERVEI_WEB;

        IF V_COUNT = 0	THEN
           RAISE EXC_SERVEI_WEB_NOEXIST;
        END IF;
      END;
      
     -- Si la regla la donem d'alta en estat Activada, verifiquem que el servei 
     -- web no tingui ja associada una regla activa.
     BEGIN
       IF (P_ESTAT= 'A')  THEN
           V_COUNT := 0;
           SELECT COUNT(*)
             INTO V_COUNT
             FROM REGLA
            WHERE SERVEI_WEB  = P_SERVEI_WEB
              AND ESTAT				= P_ESTAT;          
    
            IF V_COUNT > 0	THEN
               RAISE EXC_SERVEI_WEB_REGLA_ACTIVA;
            END IF;
          END IF;
      END;
      
      -- Insertem la regla.
      INSERT INTO REGLA 
           (DATA_ALTA, SERVEI_WEB, NOM, PERIODICITAT, ESTAT, DATA_ESTAT)
      VALUES(P_DATA_ALTA, P_SERVEI_WEB, P_NOM, P_PERIODICITAT, P_ESTAT, P_DATA_ESTAT);

       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

	      INSERT INTO LOG (DATA_HORA, NOM_PROCEDIMENT, PARAM_ENTRADA, PARAM_SORTIDA)
	           VALUES(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

        COMMIT;

      WHEN EXC_REGLA_EXIST THEN
        RSP := 'ERROR: Ja existeix una Regla per al Servei Web: ' || P_SERVEI_WEB || ' amb el Nom: ' || P_NOM;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_SERVEI_WEB_NOEXIST THEN
        RSP := 'ERROR: El Servei Web amb identificador: ' || P_SERVEI_WEB || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_SERVEI_WEB_REGLA_ACTIVA THEN
        RSP := 'ERROR: El Servei Web amb identificador: ' || P_SERVEI_WEB || ' ja t associada una regla de monitoritzaci activa';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_ALTA_REGLA;
   
  /******************************************************************************
     Nom:                 SP_BAIXA_REGLA
     Propsit:            Dona de baixa logica una regla de monitoritzaci. 
                     
     Parmetres entrada:  -	P_ID_REGLA NUMBER(10): identificaci de la regla.
                          -	P_DATA_ESTAT DATE: data d'inici de l'estat de la regla. 
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_BAIXA_REGLA (P_ID_REGLA     IN REGLA.ID_REGLA%TYPE,
                             P_DATA_ESTAT		IN REGLA.DATA_ESTAT%TYPE,
                             RSP            OUT VARCHAR2) 
    IS

   EXC_CAMP_NULL         	EXCEPTION;
   EXC_REGLA_NOEXIST			EXCEPTION;
   EXC_REGLA_BAIXA				EXCEPTION;

   V_COUNT								NUMBER;
   V_ERR_NUM       				NUMBER;
   
   V_PARAM_ENTRADA 				VARCHAR2(500);
   V_ERR_TXT       				VARCHAR2(100);
   V_CAMP_NULL      			VARCHAR2(20);
   
   V_NOM_PKG             	VARCHAR2(50):= 'PKG_REGLA.SP_BAIXA_REGLA';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_REGLA: '||P_ID_REGLA||', P_DATA_ESTAT: '||P_DATA_ESTAT;      

     -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_ID_REGLA IS NULL)  THEN
         V_CAMP_NULL := 'P_ID_REGLA';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_DATA_ESTAT IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     -- Verifiquem que la regla existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGLA
        WHERE ID_REGLA = P_ID_REGLA;

        IF V_COUNT = 0	THEN
           RAISE EXC_REGLA_NOEXIST;
        END IF;
      END;
      
     -- Verifiquem que la regla no estigui actualment donada de baixa.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGLA
        WHERE ID_REGLA = P_ID_REGLA
          AND	ESTAT		 = 'B';

        IF V_COUNT > 0	THEN
           RAISE EXC_REGLA_BAIXA;
        END IF;
      END;
      
      -- Donem de baixa lgica la regla.
      UPDATE REGLA SET 
      			 ESTAT 			= 'B',
      			 DATA_ESTAT = P_DATA_ESTAT
      WHERE  ID_REGLA 	= P_ID_REGLA;

       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGLA_NOEXIST THEN
        RSP := 'ERROR: La regla amb identificador: ' || P_ID_REGLA || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGLA_BAIXA THEN
        RSP := 'ERROR: La regla amb identificador: ' || P_ID_REGLA || ' ja est donada de baixa';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_BAIXA_REGLA;
   
  /******************************************************************************
     Nom:                 SP_MODIFICA_REGLA
     Propsit:            Actualitza una regla de monitoritzaci (taula REGLA). 
                     
     Parmetres entrada:  -	P_ID_REGLA NUMBER(10): identificaci de la regla.
                          -	P_DATA_ALTA DATE: data d'alta de la regla.
                          -	P_SERVEI_WEB NUMBER(10): servei web al que pertany la regla.
                          -	P_NOM  VARCHAR2(150): nom de la regla.
                          -	P_PERIODICITAT NUMBER(10): periodicitat d'execuci de la regla, en segons. 
                          -	P_ESTAT CHAR(1): estat de la regla (A, D). 
                          -	P_DATA_ESTAT DATE: data d'inici de l'estat de la regla. 
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
  PROCEDURE SP_MODIFICA_REGLA	 (P_ID_REGLA	      IN REGLA.ID_REGLA%TYPE,
                            	 	P_DATA_ALTA	      IN REGLA.DATA_ALTA%TYPE,
                            	 	P_SERVEI_WEB      IN REGLA.SERVEI_WEB%TYPE,
                            	 	P_NOM      				IN REGLA.NOM%TYPE,
                            		P_PERIODICITAT    IN REGLA.PERIODICITAT%TYPE,
                            		P_ESTAT					  IN REGLA.ESTAT%TYPE,
                            		P_DATA_ESTAT		  IN REGLA.DATA_ESTAT%TYPE,
                            		RSP               OUT VARCHAR2) 
    IS

   EXC_CAMP_NULL              	EXCEPTION;
   EXC_REGLA_EXIST				      EXCEPTION;
   EXC_SERVEI_WEB_NOEXIST	      EXCEPTION;
   EXC_REGLA_NOEXIST			      EXCEPTION;
   EXC_SERVEI_WEB_REGLA_ACTIVA  EXCEPTION;

   V_COUNT								      NUMBER;
   V_ERR_NUM       				      NUMBER;
   
   V_PARAM_ENTRADA 				      VARCHAR2(500);
   V_ERR_TXT       				      VARCHAR2(100);
   V_CAMP_NULL      			      VARCHAR2(20);
   
   V_NOM_PKG             	      VARCHAR2(50):= 'PKG_REGLA.SP_MODIFICA_REGLA';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_REGLA '||P_ID_REGLA||', P_DATA_ALTA '||P_DATA_ALTA||
     											', P_SERVEI_WEB: '||P_SERVEI_WEB||', P_NOM: '||P_NOM||
     											', P_PERIODICITAT: '||P_PERIODICITAT||', P_ESTAT: '||P_ESTAT||
     											', P_DATA_ESTAT: '||P_DATA_ESTAT;      

      -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_ID_REGLA IS NULL)  THEN
         V_CAMP_NULL := 'ID_REGLA';
         RAISE EXC_CAMP_NULL;
      END IF;
      
      IF (P_DATA_ALTA IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ALTA';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_SERVEI_WEB IS NULL)  THEN
         V_CAMP_NULL := 'SERVEI_WEB';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_NOM) IS NULL)  THEN
         V_CAMP_NULL := 'NOM';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_PERIODICITAT IS NULL)  THEN
         V_CAMP_NULL := 'PERIODICITAT';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_ESTAT) IS NULL)  THEN
         V_CAMP_NULL := 'ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_DATA_ESTAT IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     -- Verifiquem que la regla web existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGLA
        WHERE ID_REGLA = P_ID_REGLA;

        IF V_COUNT = 0	THEN
           RAISE EXC_REGLA_NOEXIST;
        END IF;
      END;
      
     -- Verifiquem que el servei web existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM SERVEI_WEB
        WHERE ID_SERVEI_WEB = P_SERVEI_WEB;

        IF V_COUNT = 0	THEN
           RAISE EXC_SERVEI_WEB_NOEXIST;
        END IF;
      END;      
   
      -- Verifiquem que no existeixi ja una regla, diferent a la que volem actualitzar,
      -- amb la mateixa clau.
     BEGIN
       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGLA
        WHERE SERVEI_WEB  = 	P_SERVEI_WEB
          AND NOM         = 	P_NOM          
          AND ID_REGLA  	<> 	P_ID_REGLA;          
                  
        IF V_COUNT > 0	THEN
        	RAISE EXC_REGLA_EXIST;
				END IF;
     END;
     
     -- Si la regla est en estat Activada, verifiquem que el servei 
     -- web no tingui ja associada una regla activa.
     BEGIN
       IF (P_ESTAT= 'A')  THEN
           V_COUNT := 0;
           SELECT COUNT(*)
             INTO V_COUNT
             FROM REGLA
            WHERE SERVEI_WEB  =   P_SERVEI_WEB
              AND ESTAT				=   P_ESTAT          
              AND ID_REGLA    <> 	P_ID_REGLA;          
    
            IF V_COUNT > 0	THEN
               RAISE EXC_SERVEI_WEB_REGLA_ACTIVA;
            END IF;
          END IF;
      END;
      
      -- Actualitzem la regla.
     UPDATE REGLA 
        SET  DATA_ALTA      = P_DATA_ALTA,
             SERVEI_WEB     = P_SERVEI_WEB,
             NOM   					= P_NOM,
             PERIODICITAT		= P_PERIODICITAT,
             ESTAT    			= P_ESTAT,                                                                  
             DATA_ESTAT     = P_DATA_ESTAT
     WHERE   ID_REGLA	= P_ID_REGLA;      
      
       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGLA_NOEXIST THEN
        RSP := 'ERROR: La Regla amb identificador: ' || P_ID_REGLA || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_SERVEI_WEB_NOEXIST THEN
        RSP := 'ERROR: El Servei Web amb identificador: ' || P_SERVEI_WEB || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGLA_EXIST THEN
        RSP := 'ERROR: Ja existeix una Regla associada al Servei Web: ' || P_SERVEI_WEB || ' amb el Nom: ' || P_NOM;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_SERVEI_WEB_REGLA_ACTIVA THEN
        RSP := 'ERROR: El Servei Web amb identificador: ' || P_SERVEI_WEB || ' ja t associat una regla de monitoritzaci activa';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_MODIFICA_REGLA;
   
END PKG_REGLA;
/
spool off;
