/*
* Creaci del package PKG_SERVEI_WEB.
* S'ha d'executar connectat com a monitor/monitor1606
*/

spool 15_PackageServeiWeb.log;

CREATE OR REPLACE PACKAGE PKG_SERVEI_WEB AS

  -- Procediment per donar d'alta un Servei Web. 
  PROCEDURE SP_ALTA_SERVEI_WEB (P_CLIENT         	IN SERVEI_WEB.CLIENT%TYPE,
                            		P_DESCRIPCIO      IN SERVEI_WEB.DESCRIPCIO%TYPE,
                            		P_DATA_ALTA       IN SERVEI_WEB.DATA_ALTA%TYPE,
                            		P_TIPUS_SERVEI    IN SERVEI_WEB.TIPUS_SERVEI%TYPE,
                            		P_URL				      IN SERVEI_WEB.URL%TYPE,
                            		P_ESTAT					  IN SERVEI_WEB.ESTAT%TYPE,
                            		P_DATA_ESTAT			IN SERVEI_WEB.DATA_ESTAT%TYPE,
                            		RSP               OUT VARCHAR2);
                                 
	-- Procediment per donar de baixa lgica el Servei Web, a partir de l'identificador passat per parametre.
  PROCEDURE SP_BAIXA_SERVEI_WEB (P_ID_SERVEI_WEB 	IN SERVEI_WEB.ID_SERVEI_WEB%TYPE,
                            		 P_DATA_ESTAT			IN SERVEI_WEB.DATA_ESTAT%TYPE,
  															 RSP              OUT VARCHAR2);

	-- Procediment per actualitzar el Servei Web.
  PROCEDURE SP_MODIFICA_SERVEI_WEB (P_ID_SERVEI_WEB   IN SERVEI_WEB.ID_SERVEI_WEB%TYPE,
                                    P_CLIENT          IN SERVEI_WEB.CLIENT%TYPE,
				                            P_DESCRIPCIO      IN SERVEI_WEB.DESCRIPCIO%TYPE,
                                    P_DATA_ALTA       IN SERVEI_WEB.DATA_ALTA%TYPE,
				                            P_TIPUS_SERVEI    IN SERVEI_WEB.TIPUS_SERVEI%TYPE,
				                            P_URL				     	IN SERVEI_WEB.URL%TYPE,
				                            P_ESTAT					 	IN SERVEI_WEB.ESTAT%TYPE,
                                    P_DATA_ESTAT			IN SERVEI_WEB.DATA_ESTAT%TYPE,
				                            RSP               OUT VARCHAR2); 

END PKG_SERVEI_WEB;
/


CREATE OR REPLACE PACKAGE BODY PKG_SERVEI_WEB AS

  /******************************************************************************
     Nom:                 SP_ALTA_SERVEI_WEB
     Propsit:            Dona d'alta un servei web a la taula SERVEI_WEB. 
                     
     Parmetres entrada:  -	P_CLIENT NUMBER(10): identificaci del client propietari.
                          -	P_DESCRIPCIO VARCHAR2(150): descripci del servei web.
                          -	P_DATA_ALTA DATE: data d'alta del servei web.
                          -	P_TIPUS_SERVEI  VARCHAR2(1): tipus de servei web (H, R, S).
                          -	P_URL CHAR(200): url principal del servei web. 
                          -	P_ESTAT CHAR(1): estat del servei web (A, D, N). 
                          -	P_DATA_ESTAT DATE: data d'inici de l'estat del servei web.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_ALTA_SERVEI_WEB (P_CLIENT          IN SERVEI_WEB.CLIENT%TYPE,
                            		 P_DESCRIPCIO      IN SERVEI_WEB.DESCRIPCIO%TYPE,
                            		 P_DATA_ALTA       IN SERVEI_WEB.DATA_ALTA%TYPE,
                            		 P_TIPUS_SERVEI    IN SERVEI_WEB.TIPUS_SERVEI%TYPE,
                            		 P_URL				     IN SERVEI_WEB.URL%TYPE,
                            		 P_ESTAT					 IN SERVEI_WEB.ESTAT%TYPE,
                            		 P_DATA_ESTAT			 IN SERVEI_WEB.DATA_ESTAT%TYPE,
                            		 RSP               OUT VARCHAR2) 
    IS

   EXC_CAMP_NULL         	EXCEPTION;
   EXC_SERVEI_WEB_EXIST		EXCEPTION;
   EXC_CLIENT_NOEXIST			EXCEPTION;

   V_COUNT								NUMBER;
   V_ERR_NUM       				NUMBER;
   
   V_PARAM_ENTRADA 				VARCHAR2(500);
   V_ERR_TXT       				VARCHAR2(100);
   V_CAMP_NULL      			VARCHAR2(20);
   
   V_NOM_PKG             	VARCHAR2(50):= 'PKG_SERVEI_WEB.SP_ALTA_SERVEI_WEB';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_CLIENT '||P_CLIENT||', P_DESCRIPCIO: '||P_DESCRIPCIO||
                      		', P_DATA_ALTA: '||P_DATA_ALTA||', P_TIPUS_SERVEI: '||P_TIPUS_SERVEI||
                          ', P_URL: '||P_URL||', P_ESTAT: '||P_ESTAT||', P_DATA_ESTAT: '||P_DATA_ESTAT;      

     -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_CLIENT IS NULL)  THEN
         V_CAMP_NULL := 'P_CLIENT';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (TRIM(P_DESCRIPCIO) IS NULL)  THEN
         V_CAMP_NULL := 'P_DESCRIPCIO';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_DATA_ALTA IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ALTA';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (TRIM(P_TIPUS_SERVEI) IS NULL)  THEN
         V_CAMP_NULL := 'P_TIPUS_SERVEI';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_URL) IS NULL)  THEN
         V_CAMP_NULL := 'P_URL';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_ESTAT) IS NULL)  THEN
         V_CAMP_NULL := 'P_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_DATA_ESTAT IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     -- Verifiquem que no existeixi ja un servei web amb la mateixa clau.
     BEGIN

       SELECT COUNT(*)
         INTO V_COUNT
         FROM SERVEI_WEB
        WHERE CLIENT      = P_CLIENT
          AND DESCRIPCIO  = P_DESCRIPCIO;          
                  
        IF V_COUNT > 0	THEN
        	RAISE EXC_SERVEI_WEB_EXIST;
				END IF;
     END;
     
     -- Verifiquem que el client existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM CLIENT
        WHERE ID_CLIENT = P_CLIENT;

        IF V_COUNT = 0	THEN
           RAISE EXC_CLIENT_NOEXIST;
        END IF;
      END;
      
      -- Insertem el servei web.
      INSERT INTO SERVEI_WEB 
           (CLIENT, DESCRIPCIO, DATA_ALTA, TIPUS_SERVEI, URL, ESTAT, DATA_ESTAT)
      VALUES(P_CLIENT, P_DESCRIPCIO, P_DATA_ALTA, P_TIPUS_SERVEI, P_URL, P_ESTAT, P_DATA_ESTAT);

       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_SERVEI_WEB_EXIST THEN
        RSP := 'ERROR: Ja existeix un Servei Web al Client: ' || P_CLIENT || ' amb la Descripci: ' || P_DESCRIPCIO;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_CLIENT_NOEXIST THEN
        RSP := 'ERROR: El Client amb identificador: ' || P_CLIENT || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_ALTA_SERVEI_WEB;
   
  /******************************************************************************
     Nom:                 SP_BAIXA_SERVEI_WEB
     Propsit:            Dona de baixa un servei web a la taula SERVEI_WEB. 
                     
     Parmetres entrada:  -	P_ID_SERVEI_WEB NUMBER(10): identificaci del servei web.
                          -	P_DATA_ESTAT DATE: data d'inici de l'estat del servei web.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_BAIXA_SERVEI_WEB (P_ID_SERVEI_WEB  IN SERVEI_WEB.ID_SERVEI_WEB%TYPE,
                                  P_DATA_ESTAT		 IN SERVEI_WEB.DATA_ESTAT%TYPE,
                            		  RSP              OUT VARCHAR2) 
    IS

   EXC_CAMP_NULL         	EXCEPTION;
   EXC_SERVEI_WEB_NOEXIST	EXCEPTION;
   EXC_SERVEI_WEB_BAIXA		EXCEPTION;

   V_COUNT								NUMBER;
   V_ERR_NUM       				NUMBER;
   
   V_PARAM_ENTRADA 				VARCHAR2(500);
   V_ERR_TXT       				VARCHAR2(100);
   V_CAMP_NULL      			VARCHAR2(20);
   
   V_NOM_PKG             	VARCHAR2(50):= 'PKG_SERVEI_WEB.SP_BAIXA_SERVEI_WEB';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_SERVEI_WEB: '||P_ID_SERVEI_WEB||', P_DATA_ESTAT: '||P_DATA_ESTAT;      

     -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_ID_SERVEI_WEB IS NULL)  THEN
         V_CAMP_NULL := 'P_ID_SERVEI_WEB';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_DATA_ESTAT IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;

     -- Verifiquem que el servei web existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM SERVEI_WEB
        WHERE ID_SERVEI_WEB = P_ID_SERVEI_WEB;

        IF V_COUNT = 0	THEN
           RAISE EXC_SERVEI_WEB_NOEXIST;
        END IF;
      END;
      
     -- Verifiquem que el servei web no estigui actualment donat de baixa.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM SERVEI_WEB
        WHERE ID_SERVEI_WEB = P_ID_SERVEI_WEB
          AND	ESTAT					= 'B';

        IF V_COUNT > 0	THEN
           RAISE EXC_SERVEI_WEB_BAIXA;
        END IF;
      END;
      
      -- Donem de baixa lgica el servei web.
      UPDATE SERVEI_WEB SET 
      			 ESTAT 					= 'B',
      			 DATA_ESTAT 		= P_DATA_ESTAT
      WHERE  ID_SERVEI_WEB 	= P_ID_SERVEI_WEB;

       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_SERVEI_WEB_NOEXIST THEN
        RSP := 'ERROR: El Servei Web amb identificador: ' || P_ID_SERVEI_WEB || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_SERVEI_WEB_BAIXA THEN
        RSP := 'ERROR: El Servei Web amb identificador: ' || P_ID_SERVEI_WEB || ' ja est donat de baixa';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_BAIXA_SERVEI_WEB;
   
  /******************************************************************************
     Nom:                 SP_MODIFICA_SERVEI_WEB
     Propsit:            Actualitza un servei web (taula SERVEI_WEB). 
                     
     Parmetres entrada:  -	P_ID_SERVEI_WEB NUMBER(10): identificaci del servei web.
                          -	P_CLIENT: NUMBER(10): identificaci del client propietari.
                          -	P_DESCRIPCIO VARCHAR2(150): descripci del servei web.
                          -	P_DATA_ALTA DATE: data d'alta del servei web.
                          -	P_TIPUS_SERVEI  VARCHAR2(1): tipus de servei web (H, R, S).
                          -	P_URL CHAR(200): url principal del servei web. 
                          -	P_ESTAT CHAR(1): estat del servei web (A, D, N). 
                          -	P_DATA_ESTAT DATE: data d'inici de l'estat del servei web.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_MODIFICA_SERVEI_WEB (P_ID_SERVEI_WEB   IN SERVEI_WEB.ID_SERVEI_WEB%TYPE,
                                     P_CLIENT          IN SERVEI_WEB.CLIENT%TYPE,
		                            		 P_DESCRIPCIO      IN SERVEI_WEB.DESCRIPCIO%TYPE,
                                     P_DATA_ALTA       IN SERVEI_WEB.DATA_ALTA%TYPE,
                                     P_TIPUS_SERVEI    IN SERVEI_WEB.TIPUS_SERVEI%TYPE,
                                     P_URL				     IN SERVEI_WEB.URL%TYPE,
                                     P_ESTAT					 IN SERVEI_WEB.ESTAT%TYPE,
                                     P_DATA_ESTAT			 IN SERVEI_WEB.DATA_ESTAT%TYPE,
		                            		 RSP               OUT VARCHAR2) 
    IS

   EXC_CAMP_NULL         	EXCEPTION;
   EXC_SERVEI_WEB_EXIST		EXCEPTION;
   EXC_CLIENT_NOEXIST	    EXCEPTION;
   EXC_SERVEI_WEB_NOEXIST	EXCEPTION;

   V_COUNT								NUMBER;
   V_ERR_NUM       				NUMBER;
   
   V_PARAM_ENTRADA 				VARCHAR2(500);
   V_ERR_TXT       				VARCHAR2(100);
   V_CAMP_NULL      			VARCHAR2(20);
   
   V_NOM_PKG             	VARCHAR2(50):= 'PKG_SERVEI_WEB.SP_ALTA_SERVEI_WEB';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_SERVEI_WEB: '||P_ID_SERVEI_WEB||', P_CLIENT '||P_CLIENT||', P_DESCRIPCIO: '||P_DESCRIPCIO||
                      		', P_DATA_ALTA: '||P_DATA_ALTA||', P_TIPUS_SERVEI: '||P_TIPUS_SERVEI||
                          ', P_URL: '||P_URL||', P_ESTAT: '||P_ESTAT||', P_DATA_ESTAT: '||P_DATA_ESTAT;      

      -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_ID_SERVEI_WEB IS NULL)  THEN
         V_CAMP_NULL := 'P_ID_SERVEI_WEB';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_CLIENT IS NULL)  THEN
         V_CAMP_NULL := 'P_CLIENT';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (TRIM(P_DESCRIPCIO) IS NULL)  THEN
         V_CAMP_NULL := 'P_DESCRIPCIO';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_DATA_ALTA IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ALTA';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (TRIM(P_TIPUS_SERVEI) IS NULL)  THEN
         V_CAMP_NULL := 'P_TIPUS_SERVEI';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_URL) IS NULL)  THEN
         V_CAMP_NULL := 'P_URL';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (TRIM(P_ESTAT) IS NULL)  THEN
         V_CAMP_NULL := 'P_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_DATA_ESTAT IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_ESTAT';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     -- Verifiquem que el servei web existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM SERVEI_WEB
        WHERE ID_SERVEI_WEB = P_ID_SERVEI_WEB;

        IF V_COUNT = 0	THEN
           RAISE EXC_SERVEI_WEB_NOEXIST;
        END IF;
      END;
      
      -- Verifiquem que no existeixi ja un servei web, diferent al que volem actualitzar,
      -- amb la mateixa clau.
     BEGIN
       SELECT COUNT(*)
         INTO V_COUNT
         FROM SERVEI_WEB
        WHERE CLIENT      		= 	P_CLIENT
          AND DESCRIPCIO  		= 	P_DESCRIPCIO          
          AND ID_SERVEI_WEB  	<> 	P_ID_SERVEI_WEB;          
                  
        IF V_COUNT > 0	THEN
        	RAISE EXC_SERVEI_WEB_EXIST;
				END IF;
     END;
     
     -- Verifiquem que el client existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM CLIENT
        WHERE ID_CLIENT = P_CLIENT;

        IF V_COUNT = 0	THEN
           RAISE EXC_CLIENT_NOEXIST;
        END IF;
      END;      
   
      -- Actualitzem el servei web.
     UPDATE SERVEI_WEB 
        SET  CLIENT       	= P_CLIENT,
             DESCRIPCIO   	= P_DESCRIPCIO,
             DATA_ALTA   	  = P_DATA_ALTA,
             TIPUS_SERVEI		= P_TIPUS_SERVEI,
             URL        		= P_URL,
             ESTAT    			= P_ESTAT,                                                                  
             DATA_ESTAT    	= P_DATA_ESTAT                                                                  
     WHERE   ID_SERVEI_WEB	= P_ID_SERVEI_WEB;      
      
       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_SERVEI_WEB_NOEXIST THEN
        RSP := 'ERROR: El Servei Web amb identificador: ' || P_ID_SERVEI_WEB || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_CLIENT_NOEXIST THEN
        RSP := 'ERROR: El Client amb identificador: ' || P_CLIENT || ' no existeix';

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_SERVEI_WEB_EXIST THEN
        RSP := 'ERROR: Ja existeix un Servei Web al Client: ' || P_CLIENT || ' amb la Descripci: ' || P_DESCRIPCIO;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

	      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_MODIFICA_SERVEI_WEB;
   
END PKG_SERVEI_WEB;
/
spool off;
