/*
* Creaci del package PKG_LINIA_REGISTRE.
* S'ha d'executar connectat com a monitor/monitor1606
*/

spool 16_PackageLiniaRegistre.log;

CREATE OR REPLACE PACKAGE PKG_LINIA_REGISTRE AS

  -- Procediment per donar d'alta una Linia de Registre de Monitoritzaci. 
  PROCEDURE SP_ALTA_LINIA_REGISTRE 		 (P_REGISTRE      			IN LINIA_REGISTRE.REGISTRE%TYPE,
					                          		P_PAS      						IN LINIA_REGISTRE.PAS%TYPE,
					                          		P_DATA_INICI      		IN LINIA_REGISTRE.DATA_INICI%TYPE,
					                          		P_DATA_FI      				IN LINIA_REGISTRE.DATA_FI%TYPE,
					                          		P_TEMPS_RESPOSTA			IN LINIA_REGISTRE.TEMPS_RESPOSTA%TYPE,
					                          		P_IND_DISPONIBLE    	IN LINIA_REGISTRE.IND_DISPONIBLE%TYPE,
					                          		P_CODI_ERROR				  IN LINIA_REGISTRE.CODI_ERROR%TYPE,
					                          		RSP               		OUT VARCHAR2);
                                 
	-- Procediment per donar de baixa una Linia de Registre de Monitoritzaci, a partir de 
	-- l'identificador passat per parametre.
  PROCEDURE SP_BAIXA_LINIA_REGISTRE 	 (P_ID_LINIA_REGISTRE		IN LINIA_REGISTRE.ID_LINIA_REGISTRE%TYPE,
  														 			 		RSP            				OUT VARCHAR2);

END PKG_LINIA_REGISTRE;
/


CREATE OR REPLACE PACKAGE BODY PKG_LINIA_REGISTRE AS

  /******************************************************************************
     Nom:                 SP_ALTA_LINIA_REGISTRE
     Propsit:            Dona d'alta una Linia de Registre a la taula LINIA_REGISTRE. 
                     
     Parmetres entrada:  -	P_REGISTRE NUMBER(10): identificaci del registre.
                          -	P_PAS NUMBER(10): identificaci del pas executat.
                          -	P_DATA_INICI  DATE: data d'inici de l'execuci.
                          -	P_DATA_FI  DATE: data fi de l'execuci.
                          -	P_TEMPS_RESPOSTA  NUMBER(10): temps de resposta del pas, en milisegons.
                          -	P_IND_DISPONIBLE  CHAR(1): indica si el servei web estava disponible (S, N, null).
                          -	P_CODI_ERROR VARCHAR2(3): codi error, en cas de que succeeixi. 
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
                            		 
  PROCEDURE SP_ALTA_LINIA_REGISTRE 		 (P_REGISTRE      			IN LINIA_REGISTRE.REGISTRE%TYPE,
					                          		P_PAS      						IN LINIA_REGISTRE.PAS%TYPE,
					                          		P_DATA_INICI      		IN LINIA_REGISTRE.DATA_INICI%TYPE,
					                          		P_DATA_FI      				IN LINIA_REGISTRE.DATA_FI%TYPE,
					                          		P_TEMPS_RESPOSTA			IN LINIA_REGISTRE.TEMPS_RESPOSTA%TYPE,
					                          		P_IND_DISPONIBLE    	IN LINIA_REGISTRE.IND_DISPONIBLE%TYPE,
					                          		P_CODI_ERROR				  IN LINIA_REGISTRE.CODI_ERROR%TYPE,
					                          		RSP               		OUT VARCHAR2)
  IS


   EXC_CAMP_NULL         		EXCEPTION;
   EXC_NO_CODI_ERROR     		EXCEPTION;
   EXC_LINIA_REGISTRE_EXIST	EXCEPTION;
   EXC_REGISTRE_NOEXIST			EXCEPTION;
   EXC_PAS_NOEXIST					EXCEPTION;
   EXC_PAS_NO_REGLA					EXCEPTION;

   V_COUNT									NUMBER;
   V_REGLA									NUMBER;
   V_ERR_NUM       					NUMBER;
   V_TEMPS_RESPOSTA					NUMBER;
   
   V_PARAM_ENTRADA 					VARCHAR2(500);
   V_ERR_TXT       					VARCHAR2(100);
   V_CAMP_NULL      				VARCHAR2(20);
   
   V_NOM_PKG             		VARCHAR2(50):= 'PKG_LINIA_REGISTRE.SP_ALTA_LINIA_REGISTRE';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_REGISTRE '||P_REGISTRE||', P_PAS: '||P_PAS||', P_DATA_INICI: '||P_DATA_INICI||
                      		', P_DATA_FI: '||P_DATA_FI||', P_TEMPS_RESPOSTA: '||P_TEMPS_RESPOSTA||
                      		', P_IND_DISPONIBLE: '||P_IND_DISPONIBLE||', P_CODI_ERROR: '||P_CODI_ERROR;      

     -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_REGISTRE IS NULL)  THEN
         V_CAMP_NULL := 'P_REGISTRE';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_PAS IS NULL)  THEN
         V_CAMP_NULL := 'P_PAS';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_DATA_INICI IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_INICI';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_DATA_FI IS NULL)  THEN
         V_CAMP_NULL := 'P_DATA_FI';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_TEMPS_RESPOSTA IS NULL)  THEN
         V_CAMP_NULL := 'P_TEMPS_RESPOSTA';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF (P_IND_DISPONIBLE IS NULL)  THEN
         V_CAMP_NULL := 'P_IND_DISPONIBLE';
         RAISE EXC_CAMP_NULL;
      END IF;

     IF P_IND_DISPONIBLE = 'N' THEN
        IF (P_CODI_ERROR IS NULL)  THEN
           V_CAMP_NULL := 'P_CODI_ERROR';
           RAISE EXC_CAMP_NULL;
        END IF;      
      END IF;

     -- Si el registre esta disponible, no ha de venir codi d'error.
     IF P_IND_DISPONIBLE = 'S' THEN
        IF (P_CODI_ERROR IS NOT NULL)  THEN
           RAISE EXC_NO_CODI_ERROR;
        END IF;      
      END IF;

     -- Verifiquem que no existeixi ja una linia de registre amb la mateixa clau.
     BEGIN

       SELECT COUNT(*)
         INTO V_COUNT
         FROM LINIA_REGISTRE
        WHERE REGISTRE		= P_REGISTRE
          AND PAS  				= P_PAS
          AND DATA_INICI	= P_DATA_INICI;          
                  
        IF V_COUNT > 0	THEN
        	RAISE EXC_LINIA_REGISTRE_EXIST;
				END IF;
     END;
     
     -- Verifiquem que el registre existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGISTRE
        WHERE ID_REGISTRE = P_REGISTRE;

        IF V_COUNT = 0	THEN
           RAISE EXC_REGISTRE_NOEXIST;
        END IF;
      END;
      
     -- Verifiquem que el pas existeixi i estigui Actiu.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM PAS
        WHERE ID_PAS 	= P_PAS
          AND	ESTAT	  = 'A';
          
        IF V_COUNT = 0	THEN
           RAISE EXC_PAS_NOEXIST;
        END IF;
      END;

     -- Verifiquem que el pas pertany a la regla del registre.
     BEGIN
     
       SELECT REGLA
         INTO V_REGLA
         FROM REGISTRE
        WHERE ID_REGISTRE = P_REGISTRE;
      
       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM PAS
        WHERE ID_PAS = P_PAS
          AND REGLA = V_REGLA;

        IF V_COUNT = 0	THEN
           RAISE EXC_PAS_NO_REGLA;
        END IF;
      END;
      
      -- Calculem el temps de resposta del pas, si no ha vingut informat.
      V_TEMPS_RESPOSTA := P_TEMPS_RESPOSTA;
      IF (V_TEMPS_RESPOSTA = 0)  THEN
        V_TEMPS_RESPOSTA := ((((P_DATA_FI - P_DATA_INICI) * 24) * 60) * 60) * 1000;	
      END IF;

      -- Insertem el registre de monitoritzaci.
      INSERT INTO LINIA_REGISTRE 
           (REGISTRE, PAS, DATA_INICI, DATA_FI, TEMPS_RESPOSTA, IND_DISPONIBLE, CODI_ERROR)
      VALUES(P_REGISTRE, P_PAS, P_DATA_INICI, P_DATA_FI, V_TEMPS_RESPOSTA, P_IND_DISPONIBLE, P_CODI_ERROR);

       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_NO_CODI_ERROR THEN
        RSP := 'ERROR: El Registre amb identificador: ' || P_REGISTRE || ' est disponible per ha vingut amb un codi derror, quan no sesperava';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_LINIA_REGISTRE_EXIST THEN
        RSP := 'ERROR: Ja existeix una linia de registre pel registre: ' || P_REGISTRE || 
        ', el pas: ' || P_PAS || ' i executat en el mateix instant : ' || P_DATA_INICI;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGISTRE_NOEXIST THEN
        RSP := 'ERROR: El registre de monitoritzaci amb identificador: ' || P_REGISTRE || ' no existeix';

      WHEN EXC_PAS_NOEXIST THEN
        RSP := 'ERROR: El pas amb identificador: ' || P_PAS || ' no existeix o est Desactivat';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_PAS_NO_REGLA THEN
        RSP := 'ERROR: El pas amb identificador: ' || P_PAS || ' no pertany a la regla amb identificador: ' || V_REGLA;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_ALTA_LINIA_REGISTRE;
   
  /******************************************************************************
     Nom:                 SP_BAIXA_LINIA_REGISTRE
     Propsit:            Dona de baixa una linia de registre de la taula LINIA_REGISTRE. 
                     
     Parmetres entrada:  -	P_ID_LINIA_REGISTRE NUMBER(10): identificaci de la linia de registre.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_BAIXA_LINIA_REGISTRE (P_ID_LINIA_REGISTRE  	IN 		LINIA_REGISTRE.ID_LINIA_REGISTRE%TYPE,
                            					RSP            				OUT 	VARCHAR2) 
    IS

   EXC_CAMP_NULL         				EXCEPTION;
   EXC_LINIA_REGISTRE_NOEXIST		EXCEPTION;
   EXC_ANY_NO_INFERIOR         	EXCEPTION;

   V_COUNT											NUMBER;
   V_ERR_NUM       							NUMBER;
   V_ANY_REGISTRE  							NUMBER;
   
   V_PARAM_ENTRADA 							VARCHAR2(500);
   V_ERR_TXT       							VARCHAR2(100);
   V_CAMP_NULL      						VARCHAR2(20);
   
   V_NOM_PKG             				VARCHAR2(50):= 'PKG_LINIA_REGISTRE.SP_BAIXA_LINIA_REGISTRE';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_LINIA_REGISTRE: '||P_ID_LINIA_REGISTRE;      

     -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_ID_LINIA_REGISTRE IS NULL)  THEN
         V_CAMP_NULL := 'ID_LINIA_REGISTRE';
         RAISE EXC_CAMP_NULL;
      END IF;

     -- Verifiquem que la linia registre existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM LINIA_REGISTRE
        WHERE ID_LINIA_REGISTRE = P_ID_LINIA_REGISTRE;

        IF V_COUNT = 0	THEN
           RAISE EXC_LINIA_REGISTRE_NOEXIST;
        END IF;
      END;
      
     -- Nomes es poden eliminar registres d'exercicis anteriors a l'actual.
     BEGIN

      SELECT EXTRACT(YEAR FROM DATA_INICI)
        INTO V_ANY_REGISTRE
        FROM LINIA_REGISTRE
       WHERE ID_LINIA_REGISTRE 	= P_ID_LINIA_REGISTRE;

        IF V_ANY_REGISTRE >= EXTRACT(YEAR FROM SYSDATE)	THEN
           RAISE EXC_ANY_NO_INFERIOR;
        END IF;
      END;
      
      -- Donem de baixa la linia de registre.
      DELETE FROM  LINIA_REGISTRE  
      WHERE  ID_LINIA_REGISTRE 	= P_ID_LINIA_REGISTRE;

       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_LINIA_REGISTRE_NOEXIST THEN
        RSP := 'ERROR: La linia de registre amb identificador: ' || P_ID_LINIA_REGISTRE || ' no existeix';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_ANY_NO_INFERIOR THEN
        RSP := 'ERROR: La linia de registre amb identificador: ' || P_ID_LINIA_REGISTRE || ' no pertany a un any inferior a lactual';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_BAIXA_LINIA_REGISTRE;
   
END PKG_LINIA_REGISTRE;
/
spool off;
