/*
* Creaci del package PKG_REGISTRE.
* S'ha d'executar connectat com a monitor/monitor1606
*/

spool 17_PackageRegistre.log;

CREATE OR REPLACE PACKAGE PKG_REGISTRE AS

  -- Procediment per donar d'alta un Registre de Monitoritzaci. 
  PROCEDURE SP_ALTA_REGISTRE (P_SERVEI_WEB         			IN REGISTRE.SERVEI_WEB%TYPE,
                          		P_REGLA      							IN REGISTRE.REGLA%TYPE,
                          		P_DATA_INICI      				IN REGISTRE.DATA_INICI%TYPE,
                          		RSP               				OUT VARCHAR2,
                              P_ID_REGISTRE             OUT NUMBER);
                                 
	-- Procediment per donar de baixa un Registre de Monitoritzaci, a partir de 
	-- l'identificador passat per parametre.
  PROCEDURE SP_BAIXA_REGISTRE (P_ID_REGISTRE 						IN REGISTRE.ID_REGISTRE%TYPE,
  														 RSP            					OUT VARCHAR2);

	-- Procediment per actualitzar el Registre de Monitoritzaci.
  PROCEDURE SP_MODIFICA_REGISTRE (P_ID_REGISTRE   			IN REGISTRE.ID_REGISTRE%TYPE,
	                            		P_DATA_FI      				IN REGISTRE.DATA_FI%TYPE,
	                            		P_ULTIM_PAS      			IN REGISTRE.ULTIM_PAS%TYPE,
	                            		P_IND_DISPONIBLE    	IN REGISTRE.IND_DISPONIBLE%TYPE,
	                            		P_CODI_ERROR				  IN REGISTRE.CODI_ERROR%TYPE,
					                        RSP               		OUT VARCHAR2); 

END PKG_REGISTRE;
/


CREATE OR REPLACE PACKAGE BODY PKG_REGISTRE AS

  /******************************************************************************
     Nom:                 SP_ALTA_REGISTRE
     Propsit:            Dona d'alta un Registre de Monitoritzaci a la taula REGISTRE. 
                     
     Parmetres entrada:  -	P_SERVEI_WEB NUMBER(10): identificaci del servei web.
                          -	P_REGLA NUMBER(10): identificaci de la regla executada.
                          -	P_DATA_INICI  DATE: data d'inici de l'execuci.
                                                                 
     Parmetres sortida:  - RSP: Text amb el resultat de l'execuci.
                          - P_ID_REGISTRE: Identificador del registre creat.
                          
     Retorna:             RSP = 'OK' si l'execuci ha finalitzat amb xit.
                          RSP = 'ERROR: + descripcio de l'error' en cas d'error d'execuci.
                          P_ID_REGISTRE = codi identificar del registre creat, si l'execuci
                          del procediment s'ha realitzat correctament (RSP = 'OK').
  ******************************************************************************/
                            		 
  PROCEDURE SP_ALTA_REGISTRE (P_SERVEI_WEB         	IN REGISTRE.SERVEI_WEB%TYPE,
                            	P_REGLA      					IN REGISTRE.REGLA%TYPE,
                            	P_DATA_INICI      		IN REGISTRE.DATA_INICI%TYPE,
                            	RSP               		OUT VARCHAR2,
                              P_ID_REGISTRE         OUT NUMBER)                        		 
  IS


   EXC_CAMP_NULL         		EXCEPTION;
   EXC_REGISTRE_EXIST				EXCEPTION;
   EXC_SERVEI_WEB_NOEXIST		EXCEPTION;
   EXC_REGLA_NOEXIST				EXCEPTION;

   V_COUNT									NUMBER;
   V_ERR_NUM       					NUMBER;
   
   V_DATA_INICI           	DATE;
   
   V_IND_DISPONIBLE         CHAR;
   
   V_PARAM_ENTRADA 					VARCHAR2(500);
   V_ERR_TXT       					VARCHAR2(100);
   V_CAMP_NULL      				VARCHAR2(20);
   
   V_NOM_PKG             		VARCHAR2(50):= 'PKG_REGISTRE.SP_ALTA_REGISTRE';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_SERVEI_WEB '||P_SERVEI_WEB||', P_REGLA: '||P_REGLA||
                          ', P_DATA_INICI: '||P_DATA_INICI;      

     -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_SERVEI_WEB IS NULL)  THEN
         V_CAMP_NULL := 'SERVEI_WEB';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF (P_REGLA IS NULL)  THEN
         V_CAMP_NULL := 'REGLA';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     V_DATA_INICI := P_DATA_INICI;
     IF (V_DATA_INICI IS NULL)  THEN
         V_DATA_INICI := SYSDATE;
      END IF;

     -- Verifiquem que no existeixi ja un registre amb la mateixa clau.
     BEGIN

       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGISTRE
        WHERE SERVEI_WEB	= P_SERVEI_WEB
          AND REGLA  			= P_REGLA
          AND DATA_INICI	= P_DATA_INICI;          
                  
        IF V_COUNT > 0	THEN
        	RAISE EXC_REGISTRE_EXIST;
				END IF;
     END;
     
     -- Verifiquem que el servei web existeixi actiu.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM SERVEI_WEB
        WHERE ID_SERVEI_WEB = P_SERVEI_WEB
          AND	ESTAT			    = 'A';

        IF V_COUNT = 0	THEN
           RAISE EXC_SERVEI_WEB_NOEXIST;
        END IF;
      END;
      
     -- Verifiquem que la regla existeixi, i estigui activa.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGLA
        WHERE ID_REGLA 	= P_REGLA
          AND	ESTAT			= 'A';

        IF V_COUNT = 0	THEN
           RAISE EXC_REGLA_NOEXIST;
        END IF;
      END;
      
      -- Insertem el registre de monitoritzaci.
      -- Indique que es troba 'En execucio'.
      V_IND_DISPONIBLE := 'E';
 
      INSERT INTO REGISTRE 
           (SERVEI_WEB, REGLA, DATA_INICI, IND_DISPONIBLE)
      VALUES(P_SERVEI_WEB, P_REGLA, V_DATA_INICI, V_IND_DISPONIBLE);

       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      COMMIT;
      
      -- Obtenim l'identificador del registre que s'ha donat d'alta.
      P_ID_REGISTRE := PKG_FUNCIONS.FU_DB_GETCURRVAL('REGISTRE');

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGISTRE_EXIST THEN
        RSP := 'ERROR: Ja existeix un registre de monitoritzaci pel servei web: ' || P_SERVEI_WEB || 
        ', la regla: ' || P_REGLA || ' i executat en el mateix instant : ' || P_DATA_INICI;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_SERVEI_WEB_NOEXIST THEN
        RSP := 'ERROR: El servei web amb identificador: ' || P_SERVEI_WEB || ' no existeix o est Desactivat';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGLA_NOEXIST THEN
        RSP := 'ERROR: La regla de monitoritzaci amb identificador: ' || P_REGLA || ' no existeix o no est Activa';
        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_ALTA_REGISTRE;
   
  /******************************************************************************
     Nom:                 SP_BAIXA_REGISTRE
     Propsit:            Dona de baixa un Registre de Monitoritzaci a la taula REGISTRE.
                          Noms es permet si es d'un exercici anterior a l'actual.
                     
     Parmetres entrada:  -	P_ID_REGISTRE NUMBER(10): identificaci del registre e monitoritzaci.
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_BAIXA_REGISTRE (P_ID_REGISTRE  IN 	REGISTRE.ID_REGISTRE%TYPE,
                            		RSP            OUT 	VARCHAR2) 
    IS

   EXC_CAMP_NULL         	EXCEPTION;
   EXC_REGISTRE_NOEXIST		EXCEPTION;
   EXC_ANY_NO_INFERIOR    EXCEPTION;

   V_COUNT								NUMBER;
   V_ERR_NUM       				NUMBER;
   V_ANY_REGISTRE  				NUMBER;
   
   V_PARAM_ENTRADA 				VARCHAR2(500);
   V_ERR_TXT       				VARCHAR2(100);
   V_CAMP_NULL      			VARCHAR2(20);
   
   V_NOM_PKG             	VARCHAR2(50):= 'PKG_REGISTRE.SP_BAIXA_REGISTRE';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_REGISTRE: '||P_ID_REGISTRE;      

     -- Verifiquem que tots els camps obligatoris vinguin informats.
     IF (P_ID_REGISTRE IS NULL)  THEN
         V_CAMP_NULL := 'ID_REGISTRE';
         RAISE EXC_CAMP_NULL;
      END IF;

     -- Verifiquem que el registre existeixi.
     BEGIN

       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGISTRE
        WHERE ID_REGISTRE = P_ID_REGISTRE;

        IF V_COUNT = 0	THEN
           RAISE EXC_REGISTRE_NOEXIST;
        END IF;
      END;
      
     -- Nomes es poden eliminar registres d'exercicis anteriors a l'actual.
     BEGIN

       SELECT EXTRACT(YEAR FROM DATA_INICI)
         INTO V_ANY_REGISTRE
         FROM REGISTRE
        WHERE ID_REGISTRE = P_ID_REGISTRE;

        IF V_ANY_REGISTRE >= EXTRACT(YEAR FROM SYSDATE)	THEN
           RAISE EXC_ANY_NO_INFERIOR;
        END IF;
      END;     
     
     -- Previament, eliminem les linies del registre.
      DELETE FROM  LINIA_REGISTRE  
      WHERE  REGISTRE 	= P_ID_REGISTRE;     
      
      -- Eliminem el registre.
      DELETE FROM  REGISTRE  
      WHERE  ID_REGISTRE 	= P_ID_REGISTRE;

       -- Insertem el registre d'auditoria.
      RSP := 'OK';
      SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGISTRE_NOEXIST THEN
        RSP := 'ERROR: El registre amb identificador: ' || P_ID_REGISTRE || ' no existeix';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_ANY_NO_INFERIOR THEN
        RSP := 'ERROR: El registre amb identificador: ' || P_ID_REGISTRE || ' no pertany a un any inferior a lactual';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_BAIXA_REGISTRE;
   
  /******************************************************************************
     Nom:                 SP_MODIFICA_REGISTRE
     Propsit:            Actualitza un Registre de Monitoritzaci (taula REGISTRE). 
                     
     Parmetres entrada:  -	P_ID_REGISTRE NUMBER(10): identificaci del registre.
                          -	P_DATA_FI  DATE: data fi de l'execuci.
                          -	P_ULTIM_PAS  NUMBER(10): ultim pas de la regla executat.
                          -	P_IND_DISPONIBLE  CHAR(1): indica si el servei web estava disponible (E, S, N).
                          -	P_CODI_ERROR VARCHAR2(3): codi error, en cas de que succeeixi. 
                                                                 
     Parmetres sortida:  RSP
     Retorna:             'OK' si l'execuci ha finalitzat amb xit.
                          'ERROR: + descripcio de l'error' en cas d'error d'execuci.
  ******************************************************************************/
   PROCEDURE SP_MODIFICA_REGISTRE (P_ID_REGISTRE   				IN REGISTRE.ID_REGISTRE%TYPE,
			                            	P_DATA_FI      				IN REGISTRE.DATA_FI%TYPE,
			                            	P_ULTIM_PAS      			IN REGISTRE.ULTIM_PAS%TYPE,
			                            	P_IND_DISPONIBLE    	IN REGISTRE.IND_DISPONIBLE%TYPE,
			                            	P_CODI_ERROR				  IN REGISTRE.CODI_ERROR%TYPE,
			                            	RSP               		OUT VARCHAR2)                        		 
    IS

   EXC_CAMP_NULL         		EXCEPTION;
   EXC_NO_CODI_ERROR     		EXCEPTION;
   EXC_REGISTRE_NOEXIST			EXCEPTION;
   EXC_PAS_NOEXIST					EXCEPTION;
   EXC_ERROR_DW					    EXCEPTION;
   EXC_PAS_NO_REGLA         EXCEPTION;
   EXC_REGISTRE_ACTUALITZAT EXCEPTION;

   V_COUNT									NUMBER;
   V_ERR_NUM       					NUMBER;
   V_TEMPS_RESPOSTA_TOT			NUMBER;
   V_ANY						        NUMBER;
   V_MES						        NUMBER;
   V_SERVEI_WEB							NUMBER;
   V_REGLA						      NUMBER;
   V_REGISTRES_TOT         	NUMBER;
   
   V_DATA_INICI							DATE;
   V_DATA_FI								DATE;
   
   V_PARAM_ENTRADA 					VARCHAR2(500);
   V_RET_PROC             	VARCHAR2(500);
   V_ERR_TXT       					VARCHAR2(100);
   V_CAMP_NULL      				VARCHAR2(20);
   V_IND_DISPONIBLE        	CHAR;
   
   V_NOM_PKG             		VARCHAR2(50):= 'PKG_REGISTRE.SP_MODIFICA_REGISTRE';
   
   BEGIN

     V_PARAM_ENTRADA := 	'P_ID_REGISTRE '||P_ID_REGISTRE||', P_DATA_FI: '||P_DATA_FI||
                      		', P_ULTIM_PAS: '||P_ULTIM_PAS||', P_IND_DISPONIBLE: '||P_IND_DISPONIBLE||
                      		', P_CODI_ERROR: '||P_CODI_ERROR;      

      -- Verifiquem que tots els camps obligatoris vinguin informats.

     IF (P_ID_REGISTRE IS NULL)  THEN
         V_CAMP_NULL := 'P_ID_REGISTRE';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     V_DATA_FI := P_DATA_FI;
     IF (V_DATA_FI IS NULL)  THEN
         V_DATA_FI := SYSDATE;
      END IF;

     IF (TRIM(P_IND_DISPONIBLE) IS NULL)  THEN
         V_CAMP_NULL := 'P_IND_DISPONIBLE';
         RAISE EXC_CAMP_NULL;
      END IF;
      
     IF P_IND_DISPONIBLE = 'S' THEN
        IF (P_ULTIM_PAS IS NULL)  THEN
           V_CAMP_NULL := 'P_ULTIM_PAS';
           RAISE EXC_CAMP_NULL;
        END IF;      
      END IF;      
      
     IF P_IND_DISPONIBLE = 'N' THEN
        IF (P_CODI_ERROR IS NULL)  THEN
           V_CAMP_NULL := 'P_CODI_ERROR';
           RAISE EXC_CAMP_NULL;
        END IF;      
      END IF;      

     -- Si el registre esta disponible, no ha de venir codi d'error.
     IF P_IND_DISPONIBLE = 'S' THEN
        IF (P_CODI_ERROR IS NOT NULL)  THEN
           RAISE EXC_NO_CODI_ERROR;
        END IF;      
      END IF;
      
     -- Verifiquem que el registre existeixi.
     BEGIN
       V_COUNT := 0;
       SELECT COUNT(*)
         INTO V_COUNT
         FROM REGISTRE
        WHERE ID_REGISTRE = P_ID_REGISTRE;

        IF V_COUNT = 0	THEN
           RAISE EXC_REGISTRE_NOEXIST;
        END IF;
      END;
   
     -- Verifiquem que el pas existeixi, en cas de que el registre
     -- s'hagi executat correctament.
	     BEGIN
        IF P_IND_DISPONIBLE = 'S' THEN
           V_COUNT := 0;
           SELECT COUNT(*)
             INTO V_COUNT
             FROM PAS
            WHERE ID_PAS = P_ULTIM_PAS;
    
            IF V_COUNT = 0	THEN
               RAISE EXC_PAS_NOEXIST;
            END IF;
          END IF;
      END;
      
     -- Obtenim dades del registre.
     SELECT DATA_INICI,
            SERVEI_WEB,
            REGLA,
            IND_DISPONIBLE
       INTO V_DATA_INICI,
            V_SERVEI_WEB,
            V_REGLA,
            V_IND_DISPONIBLE
       FROM REGISTRE
      WHERE ID_REGISTRE = P_ID_REGISTRE;

     -- Verifiquem que el pas pertany a la regla.
     BEGIN
      IF P_ULTIM_PAS > 0  THEN
         V_COUNT := 0;
         SELECT COUNT(*)
           INTO V_COUNT
           FROM PAS
          WHERE ID_PAS = P_ULTIM_PAS
            AND REGLA  = V_REGLA;
  
          IF V_COUNT = 0	THEN
             RAISE EXC_PAS_NO_REGLA;
          END IF;
        END IF;
     END;
     
     -- No es pot modificar un registre ja actualitzat.
     BEGIN
      IF P_IND_DISPONIBLE <> 'E'  THEN
          IF V_IND_DISPONIBLE <> 'E'	THEN
             RAISE EXC_REGISTRE_ACTUALITZAT;
          END IF;
        END IF;
     END;     

     -- Calculem el temps d'execucio total (en milisegons).
    V_TEMPS_RESPOSTA_TOT := ((((V_DATA_FI - V_DATA_INICI) * 24) * 60) * 60) * 1000;	
      
	   -- Actualitzem el servei web.
     UPDATE REGISTRE 
        SET  DATA_FI       			= V_DATA_FI,
             TEMPS_RESPOSTA_TOT = V_TEMPS_RESPOSTA_TOT,
             ULTIM_PAS   				= P_ULTIM_PAS,
             IND_DISPONIBLE			= P_IND_DISPONIBLE,
             CODI_ERROR        	= P_CODI_ERROR                                                       
     WHERE   ID_REGISTRE				= P_ID_REGISTRE;      

	   --------------------------------------------------------------------------------------
	   -- Si ha variat l'estat del registre, actualitzem les estadistiques.
     IF (V_IND_DISPONIBLE = 'E') AND (P_IND_DISPONIBLE <> 'E') THEN
     
        -- Estadistica mensual per servei web i regla.
        PKG_TAULES_DW.SP_DISPONIBILITAT_MES(P_ID_REGISTRE, V_RET_PROC);
        IF (V_RET_PROC <> 'OK')  THEN
            RAISE EXC_ERROR_DW;
        END IF;       
        
        -- Estadistica diaria de disponibilitat, i dia amb menys disponibilitat (DW).
        PKG_TAULES_DW.SP_DISPONIBILITAT_DIA(P_ID_REGISTRE, V_RET_PROC);
        IF (V_RET_PROC <> 'OK')  THEN
            RAISE EXC_ERROR_DW;
        END IF;        
        
        -- Estadistica mensual del client amb menys disponibilitat (DW).
        PKG_TAULES_DW.SP_DW_CLIENT_MENYS_DISP(P_ID_REGISTRE, V_RET_PROC);
        IF (V_RET_PROC <> 'OK')  THEN
            RAISE EXC_ERROR_DW;
        END IF;        
       
        -- Estadistica mensual de la regla que mes temps triga en executar-se, de mitja (DW).
        PKG_TAULES_DW.SP_DW_REGLA_MES_TEMPS(P_ID_REGISTRE, V_RET_PROC);
        IF (V_RET_PROC <> 'OK')  THEN
            RAISE EXC_ERROR_DW;
        END IF;        
        
     END IF;
   
	   --------------------------------------------------------------------------------------
     -- Si tot ha anat be, insertem el registre d'auditoria i fem commit.
     RSP := 'OK';
     SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION
      WHEN EXC_CAMP_NULL THEN
        RSP := 'ERROR: El camp ' || V_CAMP_NULL || ' no pot ser nul';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_NO_CODI_ERROR THEN
        RSP := 'ERROR: El Registre amb identificador: ' || P_ID_REGISTRE || ' est disponible per ha vingut amb un codi derror, quan no sesperava';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_REGISTRE_NOEXIST THEN
        RSP := 'ERROR: El Registre amb identificador: ' || P_ID_REGISTRE || ' no existeix';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_PAS_NOEXIST THEN
        RSP := 'ERROR: El Pas amb identificador: ' || P_ULTIM_PAS || ' no existeix';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN EXC_PAS_NO_REGLA THEN
        RSP := 'ERROR: El Pas amb identificador: ' || P_ULTIM_PAS || ' no pertany a la Regla amb identificador: '||V_REGLA;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);    
        
      WHEN EXC_REGISTRE_ACTUALITZAT THEN
        RSP := 'ERROR: El Registre amb identificador: ' || P_ID_REGISTRE || ' no es pot modificar perque ja ha finalitzat la seva execuci';

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP); 
        
      WHEN EXC_ERROR_DW THEN
        RSP := V_RET_PROC;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);

      WHEN OTHERS THEN
        V_ERR_NUM := SQLCODE;
        V_ERR_TXT := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(V_ERR_NUM)||': '||V_ERR_TXT;

        SP_ALTA_LOG(SYSDATE, V_NOM_PKG, V_PARAM_ENTRADA, 'RSP: '||RSP);
        ROLLBACK;

   END SP_MODIFICA_REGISTRE;
   
END PKG_REGISTRE;
/
spool off;
